@@ -1,13 +1,13 @@
 
-#ifndef __CAAS_H
-#define __CAAS_H
+#ifndef FC_SOLVE__CAAS_H
+#define FC_SOLVE__CAAS_H
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
 /* #define FCS_USE_INLINE */
-    
+
 /*
  * check_and_add_state is defined in caas.c.
  *
@@ -15,14 +15,14 @@ extern "C" {
  * Solver uses to solve a given board.
  * */
 
-extern int freecell_solver_check_and_add_state(
-    freecell_solver_soft_thread_t * soft_thread,
-    fcs_state_with_locations_t * new_state,
-    fcs_state_with_locations_t * * existing_state
+extern int fc_solve_check_and_add_state(
+    fc_solve_soft_thread_t * soft_thread,
+    fcs_state_extra_info_t * new_state_val,
+    fcs_state_extra_info_t * * existing_state_val
     );
 
 #ifdef __cplusplus
 }
 #endif
 
-#endif /* #ifndef __CAAS_H */
+#endif /* #ifndef FC_SOLVE__CAAS_H */
