@@ -13,6 +13,8 @@
 #include "dmalloc.h"
 #endif
 
+#include "cmd_line_enum.h"
+
 static int read_preset(char * preset_name, args_man_t * * args, char * * opened_files_dir_to_assign, char * user_preset_dir)
 {
     int ret_code = 1;
@@ -92,8 +94,8 @@ static int read_preset(char * preset_name, args_man_t * * args, char * * opened_
             {
                 if (read_next_preset)
                 {
-                    *args = freecell_solver_args_man_alloc();
-                    freecell_solver_args_man_chop(*args, line+8);
+                    *args = fc_solve_args_man_alloc();
+                    fc_solve_args_man_chop(*args, line+8);
                     ret_code = 0;
                     goto HAVE_PRESET;
                 }
@@ -145,25 +147,29 @@ int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
     char * opened_files_dir
     )
 {
-    int arg;
+    char * * arg, * * arg_argc;
     char * * known_param;
     int num_to_skip;
     int callback_ret;
     int ret;
+    int opt;
+    const char * p;
 
     *error_string = NULL;
 
-    for(arg=start_arg;arg<argc;arg++)
+    arg_argc = &(argv[argc]);
+
+    for(arg=&(argv[start_arg]);arg<arg_argc;arg++)
     {
         /* First check for the parameters that the user recognizes */
         known_param = known_parameters;
-        while((*known_param) && strcmp(*known_param, argv[arg]))
+        while((*known_param) && strcmp(*known_param, (*arg)))
         {
             known_param++;
         }
         if ((*known_param) != NULL )
         {
-            callback_ret = callback(instance, argc, argv, arg, &num_to_skip, &ret, callback_context);
+            callback_ret = callback(instance, argc, argv, arg-&(argv[0]), &num_to_skip, &ret, callback_context);
             if (callback_ret == FCS_CMD_LINE_SKIP)
             {
                 arg += num_to_skip-1;
@@ -171,45 +177,664 @@ int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
             }
             else if (callback_ret == FCS_CMD_LINE_STOP)
             {
-                *last_arg = arg;
+                *last_arg = arg-&(argv[0]);
                 return ret;
             }
         }
 
-        if (0)
-        {
-        }
-        else if ((!strcmp(argv[arg], "-md")) || (!strcmp(argv[arg], "--max-depth")))
-        {
-            arg++;
-            if (arg == argc)
-            {
-                *last_arg = arg-1;
-                return FCS_CMD_LINE_PARAM_WITH_NO_ARG;
-            }
-            freecell_solver_user_limit_depth(instance, atoi(argv[arg]));
-        }
-        else if ((!strcmp(argv[arg], "-mi")) || (!strcmp(argv[arg], "--max-iters")))
-        {
-            arg++;
-            if (arg == argc)
-            {
-                *last_arg = arg-1;
-                return FCS_CMD_LINE_PARAM_WITH_NO_ARG;
-            }
+        /* OPT-PARSE-START */
+p = (*arg);
+opt = FCS_OPT_UNRECOGNIZED;
 
-            freecell_solver_user_limit_current_instance_iterations(instance, atoi(argv[arg]));
+{
+if (*(p++) == '-')
+{
+{ switch(*(p++)) {
+case '-':
+{ switch(*(p++)) {
+case 'a':
+{
+if (!strncmp(p, "-star-weights", 13)) {
+p += 13;
+opt = FCS_OPT_A_STAR_WEIGHTS;
+
+}
+}
+
+break;
+
+case 'c':
+{
+if (!strncmp(p, "alc-real-depth", 14)) {
+p += 14;
+opt = FCS_OPT_CALC_REAL_DEPTH;
+
+}
+}
+
+break;
+
+case 'd':
+{
+if (!strncmp(p, "ecks-num", 8)) {
+p += 8;
+opt = FCS_OPT_DECKS_NUM;
+
+}
+}
+
+break;
+
+case 'e':
+{
+if (!strncmp(p, "mpty-stacks-filled-by", 21)) {
+p += 21;
+opt = FCS_OPT_EMPTY_STACKS_FILLED_BY;
+
+}
+}
+
+break;
+
+case 'f':
+{
+if (!strncmp(p, "reecells-num", 12)) {
+p += 12;
+opt = FCS_OPT_FREECELLS_NUM;
+
+}
+}
+
+break;
+
+case 'g':
+{
+if (!strncmp(p, "ame", 3)) {
+p += 3;
+opt = FCS_OPT_GAME;
+
+}
+}
+
+break;
+
+case 'l':
+{
+if (!strncmp(p, "oad-config", 10)) {
+p += 10;
+opt = FCS_OPT_LOAD_CONFIG;
+
+}
+}
+
+break;
+
+case 'm':
+{ switch(*(p++)) {
+case 'a':
+{
+if (!strncmp(p, "x-", 2)) {
+p += 2;
+{ switch(*(p++)) {
+case 'd':
+{
+if (!strncmp(p, "epth", 4)) {
+p += 4;
+opt = FCS_OPT_MAX_DEPTH;
+
+}
+}
+
+break;
+
+case 'i':
+{
+if (!strncmp(p, "ters", 4)) {
+p += 4;
+opt = FCS_OPT_MAX_ITERS;
+
+}
+}
+
+break;
+
+case 's':
+{
+if (!strncmp(p, "tored-states", 12)) {
+p += 12;
+opt = FCS_OPT_MAX_STORED_STATES;
+
+}
+}
+
+break;
+
+}
+}
+
+}
+}
+
+break;
+
+case 'e':
+{
+if (!strncmp(p, "thod", 4)) {
+p += 4;
+opt = FCS_OPT_METHOD;
+
+}
+}
+
+break;
+
+}
+}
+
+break;
+
+case 'n':
+{
+if (!strncmp(p, "ext-", 4)) {
+p += 4;
+{ switch(*(p++)) {
+case 'h':
+{
+if (!strncmp(p, "ard-thread", 10)) {
+p += 10;
+opt = FCS_OPT_NEXT_HARD_THREAD;
+
+}
+}
+
+break;
+
+case 's':
+{
+if (!strncmp(p, "oft-thread", 10)) {
+p += 10;
+opt = FCS_OPT_NEXT_SOFT_THREAD;
+
+}
+}
+
+break;
+
+}
+}
+
+}
+}
+
+break;
+
+case 'o':
+{
+if (!strncmp(p, "ptimiz", 6)) {
+p += 6;
+{ switch(*(p++)) {
+case 'a':
+{
+if (!strncmp(p, "tion-tests-order", 16)) {
+p += 16;
+opt = FCS_OPT_OPTIMIZATION_TESTS_ORDER;
+
+}
+}
+
+break;
+
+case 'e':
+{
+if (!strncmp(p, "-solution", 9)) {
+p += 9;
+opt = FCS_OPT_OPTIMIZE_SOLUTION;
+
+}
+}
+
+break;
+
+}
+}
+
+}
+}
+
+break;
+
+case 'p':
+{
+if (!strncmp(p, "re", 2)) {
+p += 2;
+{ switch(*(p++)) {
+case 'l':
+{
+if (!strncmp(p, "ude", 3)) {
+p += 3;
+opt = FCS_OPT_PRELUDE;
+
+}
+}
+
+break;
+
+case 's':
+{
+if (!strncmp(p, "et", 2)) {
+p += 2;
+opt = FCS_OPT_GAME;
+
+}
+}
+
+break;
+
+}
+}
+
+}
+}
+
+break;
+
+case 'r':
+
+{
+if (*(p++) == 'e')
+{
+{ switch(*(p++)) {
+case 'a':
+{
+if (!strncmp(p, "d-from-file", 11)) {
+p += 11;
+opt = FCS_OPT_READ_FROM_FILE;
+
+}
+}
+
+break;
+
+case 'p':
+{
+if (!strncmp(p, "arent-states", 12)) {
+p += 12;
+opt = FCS_OPT_REPARENT_STATES;
+
+}
+}
+
+break;
+
+case 's':
+{
+if (!strncmp(p, "et", 2)) {
+p += 2;
+opt = FCS_OPT_RESET;
+
+}
+}
+
+break;
+
+}
+}
+
+}
+
+}
+
+break;
+
+case 's':
+{ switch(*(p++)) {
+case 'c':
+{
+if (!strncmp(p, "ans-synergy", 11)) {
+p += 11;
+opt = FCS_OPT_SCANS_SYNERGY;
+
+}
+}
+
+break;
+
+case 'e':
+{
+if (!strncmp(p, "quence", 6)) {
+p += 6;
+{ switch(*(p++)) {
+case '-':
+{
+if (!strncmp(p, "move", 4)) {
+p += 4;
+opt = FCS_OPT_SEQUENCE_MOVE;
+
+}
+}
+
+break;
+
+case 's':
+{
+if (!strncmp(p, "-are-built-by", 13)) {
+p += 13;
+opt = FCS_OPT_SEQUENCES_ARE_BUILT_BY;
+
+}
+}
+
+break;
+
+}
+}
+
+}
+}
+
+break;
+
+case 'o':
+{
+if (!strncmp(p, "ft-thread-step", 14)) {
+p += 14;
+opt = FCS_OPT_SOFT_THREAD_STEP;
+
+}
+}
+
+break;
+
+case 't':
+{ switch(*(p++)) {
+case '-':
+{
+if (!strncmp(p, "name", 4)) {
+p += 4;
+opt = FCS_OPT_ST_NAME;
+
+}
+}
+
+break;
+
+case 'a':
+{
+if (!strncmp(p, "cks-num", 7)) {
+p += 7;
+opt = FCS_OPT_STACKS_NUM;
+
+}
+}
+
+break;
+
+}
+}
+
+break;
+
+}
+}
+
+break;
+
+case 't':
+{
+if (!strncmp(p, "ests-order", 10)) {
+p += 10;
+opt = FCS_OPT_TESTS_ORDER;
+
+}
+}
+
+break;
+
+}
+}
+
+break;
+
+case 'a':
+{
+if (!strncmp(p, "sw", 2)) {
+p += 2;
+opt = FCS_OPT_A_STAR_WEIGHTS;
+
+}
+}
+
+break;
+
+case 'g':
+
+{
+if (*p == '\0')
+{
+
+opt = FCS_OPT_GAME;
+}
+}
+
+break;
+
+case 'l':
+
+{
+if (*p == '\0')
+{
+
+opt = FCS_OPT_LOAD_CONFIG;
+}
+}
+
+break;
+
+case 'm':
+{ switch(*(p++)) {
+case 'd':
+
+{
+if (*p == '\0')
+{
+
+opt = FCS_OPT_MAX_DEPTH;
+}
+}
+
+break;
+
+case 'e':
+
+{
+if (*p == '\0')
+{
+
+opt = FCS_OPT_METHOD;
+}
+}
+
+break;
+
+case 'i':
+
+{
+if (*p == '\0')
+{
+
+opt = FCS_OPT_MAX_ITERS;
+}
+}
+
+break;
+
+case 's':
+{
+if (!strncmp(p, "s", 1)) {
+p += 1;
+opt = FCS_OPT_MAX_STORED_STATES;
+
+}
+}
+
+break;
+
+}
+}
+
+break;
+
+case 'n':
+{ switch(*(p++)) {
+case 'h':
+{
+if (!strncmp(p, "t", 1)) {
+p += 1;
+opt = FCS_OPT_NEXT_HARD_THREAD;
+
+}
+}
+
+break;
+
+case 's':
+{
+if (!strncmp(p, "t", 1)) {
+p += 1;
+opt = FCS_OPT_NEXT_SOFT_THREAD;
+
+}
+}
+
+break;
+
+}
+}
+
+break;
+
+case 'o':
+{
+if (!strncmp(p, "pt", 2)) {
+p += 2;
+{ switch(*(p++)) {
+case '\0':
+{
+opt = FCS_OPT_OPTIMIZE_SOLUTION;
+}
+
+break;
+
+case '-':
+{
+if (!strncmp(p, "to", 2)) {
+p += 2;
+opt = FCS_OPT_OPTIMIZATION_TESTS_ORDER;
+
+}
+}
+
+break;
+
+}
+}
+
+}
+}
+
+break;
+
+case 's':
+{ switch(*(p++)) {
+case 'e':
+{
+if (!strncmp(p, "ed", 2)) {
+p += 2;
+opt = FCS_OPT_SEED;
+
+}
+}
+
+break;
+
+case 't':
+{
+if (!strncmp(p, "ep", 2)) {
+p += 2;
+opt = FCS_OPT_SOFT_THREAD_STEP;
+
+}
+}
+
+break;
+
+}
+}
+
+break;
+
+case 't':
+{
+if (!strncmp(p, "o", 1)) {
+p += 1;
+opt = FCS_OPT_TESTS_ORDER;
+
+}
+}
+
+break;
+
+}
+}
+
+}
+
+}
+        switch (opt)
+        {
+        case FCS_OPT_UNRECOGNIZED:
+        {
+            *last_arg = arg-&(argv[0]);
+            return FCS_CMD_LINE_UNRECOGNIZED_OPTION;
         }
-        else if ((!strcmp(argv[arg], "-to")) || (!strcmp(argv[arg], "--tests-order")))
+        break;
+
+#define PROCESS_OPT_ARG() \
+        { \
+            arg++; \
+            if (arg == arg_argc) \
+            { \
+                *last_arg = arg-&(argv[0]); \
+                return FCS_CMD_LINE_PARAM_WITH_NO_ARG; \
+            } \
+        }
+
+#define RET_ERROR_IN_ARG() \
+        *last_arg = arg-&(argv[0]); \
+        return FCS_CMD_LINE_ERROR_IN_ARG;
+
+        case FCS_OPT_MAX_DEPTH: /* STRINGS=-md|--max-depth; */
+        {
+            PROCESS_OPT_ARG() ;
+            freecell_solver_user_limit_depth(instance, atoi((*arg)));
+        }
+        break;
+
+        case FCS_OPT_MAX_ITERS: /* STRINGS=-mi|--max-iters; */
+        {
+            PROCESS_OPT_ARG() ;
+
+            freecell_solver_user_limit_current_instance_iterations(instance, atoi((*arg)));
+        }
+        break;
+
+        case FCS_OPT_TESTS_ORDER: /* STRINGS=-to|--tests-order; */
         {
             char * fcs_user_errstr;
-            arg++;
-            if (arg == argc)
-            {
-                *last_arg = arg-1;
-                return FCS_CMD_LINE_PARAM_WITH_NO_ARG;
-            }
-            ret = freecell_solver_user_set_tests_order(instance, argv[arg], &fcs_user_errstr);
+
+            PROCESS_OPT_ARG() ;
+
+            ret = freecell_solver_user_set_tests_order(instance, (*arg), &fcs_user_errstr);
             if (ret != 0)
             {
                 char * errstr = malloc(strlen(fcs_user_errstr)+500);
@@ -222,19 +847,15 @@ int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
 
                 *error_string = errstr;
 
-                *last_arg = arg;
-                return FCS_CMD_LINE_ERROR_IN_ARG;
+                RET_ERROR_IN_ARG() ;
             }
         }
-        else if ((!strcmp(argv[arg], "--freecells-num")))
+        break;
+
+        case FCS_OPT_FREECELLS_NUM: /* STRINGS=--freecells-num; */
         {
-            arg++;
-            if (arg == argc)
-            {
-                *last_arg = arg-1;
-                return FCS_CMD_LINE_PARAM_WITH_NO_ARG;
-            }
-            if (freecell_solver_user_set_num_freecells(instance, atoi(argv[arg])) != 0)
+            PROCESS_OPT_ARG() ;
+            if (freecell_solver_user_set_num_freecells(instance, atoi((*arg))) != 0)
             {
                 char * errstr;
 
@@ -248,19 +869,15 @@ int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
 
                 *error_string = errstr;
 
-                *last_arg = arg;
-                return FCS_CMD_LINE_ERROR_IN_ARG;
+                RET_ERROR_IN_ARG() ;
             }
         }
-        else if ((!strcmp(argv[arg], "--stacks-num")))
+        break;
+
+        case FCS_OPT_STACKS_NUM: /* STRINGS=--stacks-num; */
         {
-            arg++;
-            if (arg == argc)
-            {
-                *last_arg = arg-1;
-                return FCS_CMD_LINE_PARAM_WITH_NO_ARG;
-            }
-            if (freecell_solver_user_set_num_stacks(instance, atoi(argv[arg])) != 0)
+            PROCESS_OPT_ARG() ;
+            if (freecell_solver_user_set_num_stacks(instance, atoi((*arg))) != 0)
             {
                 char * errstr;
 
@@ -274,19 +891,15 @@ int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
 
                 *error_string = errstr;
 
-                *last_arg = arg;
-                return FCS_CMD_LINE_ERROR_IN_ARG;
+                RET_ERROR_IN_ARG() ;
             }
         }
-        else if ((!strcmp(argv[arg], "--decks-num")))
+        break;
+
+        case FCS_OPT_DECKS_NUM: /* STRINGS=--decks-num; */
         {
-            arg++;
-            if (arg == argc)
-            {
-                *last_arg = arg-1;
-                return FCS_CMD_LINE_PARAM_WITH_NO_ARG;
-            }
-            if (freecell_solver_user_set_num_decks(instance, atoi(argv[arg])) != 0)
+            PROCESS_OPT_ARG() ;
+            if (freecell_solver_user_set_num_decks(instance, atoi((*arg))) != 0)
             {
                 char * errstr;
 
@@ -300,26 +913,22 @@ int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
 
                 *error_string = errstr;
 
-                *last_arg = arg;
-                return FCS_CMD_LINE_ERROR_IN_ARG;
+                RET_ERROR_IN_ARG() ;
             }
         }
-        else if ((!strcmp(argv[arg], "--sequences-are-built-by")))
+        break;
+
+        case FCS_OPT_SEQUENCES_ARE_BUILT_BY: /* STRINGS=--sequences-are-built-by; */
         {
             int sbb;
 
-            arg++;
-            if (arg == argc)
-            {
-                *last_arg = arg-1;
-                return FCS_CMD_LINE_PARAM_WITH_NO_ARG;
-            }
+            PROCESS_OPT_ARG() ;
 
-            if (!strcmp(argv[arg], "suit"))
+            if (!strcmp((*arg), "suit"))
             {
                 sbb = FCS_SEQ_BUILT_BY_SUIT;
             }
-            else if (!strcmp(argv[arg], "rank"))
+            else if (!strcmp((*arg), "rank"))
             {
                 sbb = FCS_SEQ_BUILT_BY_RANK;
             }
@@ -329,18 +938,15 @@ int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
             }
             freecell_solver_user_set_sequences_are_built_by_type(instance, sbb);
         }
-        else if ((!strcmp(argv[arg], "--sequence-move")))
+        break;
+
+        case FCS_OPT_SEQUENCE_MOVE: /* STRINGS=--sequence-move; */
         {
             int unlimited;
 
-            arg++;
-            if (arg == argc)
-            {
-                *last_arg = arg-1;
-                return FCS_CMD_LINE_PARAM_WITH_NO_ARG;
-            }
+            PROCESS_OPT_ARG() ;
 
-            if (!strcmp(argv[arg], "unlimited"))
+            if (!strcmp((*arg), "unlimited"))
             {
                 unlimited = 1;
             }
@@ -350,22 +956,19 @@ int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
             }
             freecell_solver_user_set_sequence_move(instance, unlimited);
         }
-        else if (!strcmp(argv[arg], "--empty-stacks-filled-by"))
+        break;
+
+        case FCS_OPT_EMPTY_STACKS_FILLED_BY: /* STRINGS=--empty-stacks-filled-by; */
         {
             int es_fill;
 
-            arg++;
-            if (arg == argc)
-            {
-                *last_arg = arg-1;
-                return FCS_CMD_LINE_PARAM_WITH_NO_ARG;
-            }
+            PROCESS_OPT_ARG() ;
 
-            if (!strcmp(argv[arg], "kings"))
+            if (!strcmp((*arg), "kings"))
             {
                 es_fill = FCS_ES_FILLED_BY_KINGS_ONLY;
             }
-            else if (!strcmp(argv[arg], "none"))
+            else if (!strcmp((*arg), "none"))
             {
                 es_fill = FCS_ES_FILLED_BY_NONE;
             }
@@ -378,112 +981,96 @@ int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
                 es_fill
                 );
         }
-        else if (
-            (!strcmp(argv[arg], "--game")) ||
-            (!strcmp(argv[arg], "--preset")) ||
-            (!strcmp(argv[arg], "-g"))
-            )
-        {
-            arg++;
-            if (arg == argc)
-            {
-                *last_arg = arg-1;
-                return FCS_CMD_LINE_PARAM_WITH_NO_ARG;
-            }
+        break;
 
-            ret = freecell_solver_user_apply_preset(instance, argv[arg]);
+        case FCS_OPT_GAME: /* STRINGS=--game|--preset|-g; */
+        {
+            PROCESS_OPT_ARG() ;
+
+            ret = freecell_solver_user_apply_preset(instance, (*arg));
             if (ret == FCS_PRESET_CODE_NOT_FOUND)
             {
                 char * errstr;
 
-                errstr = malloc(strlen(argv[arg])+500);
+                errstr = malloc(strlen((*arg))+500);
 
-                sprintf(errstr, "Unknown game \"%s\"!\n\n", argv[arg]);
+                sprintf(errstr, "Unknown game \"%s\"!\n\n", (*arg));
                 *error_string = errstr;
 
-                *last_arg = arg;
-                return FCS_CMD_LINE_ERROR_IN_ARG;
+                RET_ERROR_IN_ARG() ;
             }
             else if (ret == FCS_PRESET_CODE_FREECELLS_EXCEED_MAX)
             {
                 char * errstr;
 
-                errstr = malloc(strlen(argv[arg])+500);
+                errstr = malloc(strlen((*arg))+500);
                 sprintf(errstr, "The game \"%s\" exceeds the maximal number "
                         "of freecells in the program.\n"
                         "Modify the file \"config.h\" and recompile, "
                         "if you wish to solve one of its boards.\n",
-                        argv[arg]
+                        (*arg)
                         );
 
                 *error_string = errstr;
 
-                *last_arg = arg;
-                return FCS_CMD_LINE_ERROR_IN_ARG;
+                RET_ERROR_IN_ARG() ;
             }
             else if (ret == FCS_PRESET_CODE_STACKS_EXCEED_MAX)
             {
                 char * errstr;
 
-                errstr = malloc(strlen(argv[arg])+500);
+                errstr = malloc(strlen((*arg))+500);
 
                 sprintf(errstr, "The game \"%s\" exceeds the maximal number "
                         "of stacks in the program.\n"
                         "Modify the file \"config.h\" and recompile, "
                         "if you wish to solve one of its boards.\n",
-                        argv[arg]
+                        (*arg)
                         );
 
                 *error_string = errstr;
 
-                *last_arg = arg;
-                return FCS_CMD_LINE_ERROR_IN_ARG;
+                RET_ERROR_IN_ARG() ;
             }
             else if (ret != FCS_PRESET_CODE_OK)
             {
                 char * errstr;
 
-                errstr = malloc(strlen(argv[arg])+500);
+                errstr = malloc(strlen((*arg))+500);
 
                 sprintf(errstr,
                     "The game \"%s\" exceeds the limits of the program.\n"
                     "Modify the file \"config.h\" and recompile, if you wish to solve one of its boards.\n",
-                    argv[arg]
+                    (*arg)
                 );
 
                 *error_string = errstr;
 
-                *last_arg = arg;
-                return FCS_CMD_LINE_ERROR_IN_ARG;
+                RET_ERROR_IN_ARG() ;
             }
         }
-        else if ((!strcmp(argv[arg], "-me")) || (!strcmp(argv[arg], "--method")))
+        break;
+
+        case FCS_OPT_METHOD: /* STRINGS=-me|--method; */
         {
             int method;
 
-            arg++;
-            if (arg == argc)
-            {
-                *last_arg = arg-1;
-                return FCS_CMD_LINE_PARAM_WITH_NO_ARG;
-            }
-            if (!strcmp(argv[arg], "dfs"))
-            {
-                method = FCS_METHOD_HARD_DFS;
-            }
-            else if (!strcmp(argv[arg], "soft-dfs"))
+            PROCESS_OPT_ARG() ;
+
+            if (!strcmp((*arg), "soft-dfs") ||
+                !strcmp((*arg), "dfs"))
             {
                 method = FCS_METHOD_SOFT_DFS;
             }
-            else if (!strcmp(argv[arg], "bfs"))
+            else if (!strcmp((*arg), "bfs"))
             {
                 method = FCS_METHOD_BFS;
             }
-            else if (!strcmp(argv[arg], "a-star"))
+            else if (!strcmp((*arg), "a-star"))
             {
                 method = FCS_METHOD_A_STAR;
             }
-            else if (!strcmp(argv[arg], "random-dfs"))
+            else if (!strcmp((*arg), "random-dfs"))
             {
                 method = FCS_METHOD_RANDOM_DFS;
             }
@@ -491,36 +1078,33 @@ int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
             {
                 char * errstr;
 
-                errstr = malloc(strlen(argv[arg])+500);
+                errstr = malloc(strlen((*arg))+500);
 
                 sprintf(
                     errstr,
                     "Unknown solving method \"%s\".\n",
-                    argv[arg]
+                    (*arg)
                     );
 
                 *error_string = errstr;
 
-                *last_arg = arg;
-                return FCS_CMD_LINE_ERROR_IN_ARG;
+                RET_ERROR_IN_ARG() ;
             }
 
             freecell_solver_user_set_solving_method(instance, method);
         }
-        else if ((!strcmp(argv[arg], "-asw")) || (!strcmp(argv[arg], "--a-star-weights")))
+        break;
+
+        case FCS_OPT_A_STAR_WEIGHTS: /* STRINGS=-asw|--a-star-weights; */
         {
-            arg++;
-            if (arg == argc)
-            {
-                *last_arg = arg-1;
-                return FCS_CMD_LINE_PARAM_WITH_NO_ARG;
-            }
+            PROCESS_OPT_ARG() ;
+
             {
                 int a;
                 char * start_num;
                 char * end_num;
                 char save;
-                start_num = argv[arg];
+                start_num = (*arg);
                 for(a=0;a<5;a++)
                 {
                     while ((*start_num > '9') && (*start_num < '0') && (*start_num != '\0'))
@@ -548,49 +1132,42 @@ int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
                 }
             }
         }
-        else if ((!strcmp(argv[arg], "-opt")) || (!strcmp(argv[arg], "--optimize-solution")))
+        break;
+
+        case FCS_OPT_OPTIMIZE_SOLUTION: /* STRINGS=-opt|--optimize-solution; */
         {
             freecell_solver_user_set_solution_optimization(instance, 1);
         }
-        else if ((!strcmp(argv[arg], "-seed")))
-        {
-            arg++;
-            if (arg == argc)
-            {
-                *last_arg = arg-1;
-                return FCS_CMD_LINE_PARAM_WITH_NO_ARG;
-            }
+        break;
 
-            freecell_solver_user_set_random_seed(instance, atoi(argv[arg]));
-        }
-        else if ((!strcmp(argv[arg], "-mss")) || (!strcmp(argv[arg], "--max-stored-states")))
+        case FCS_OPT_SEED: /* STRINGS=-seed; */
         {
-            arg++;
-            if (arg == argc)
-            {
-                *last_arg = arg-1;
-                return FCS_CMD_LINE_PARAM_WITH_NO_ARG;
-            }
+            PROCESS_OPT_ARG() ;
+
+            freecell_solver_user_set_random_seed(instance, atoi((*arg)));
+        }
+        break;
+
+        case FCS_OPT_MAX_STORED_STATES: /* STRINGS=-mss|--max-stored-states; */
+        {
+            PROCESS_OPT_ARG() ;
 
             freecell_solver_user_limit_num_states_in_collection(
                 instance,
-                atoi(argv[arg])
+                atoi((*arg))
             );
         }
-        else if (
-            (!strcmp(argv[arg], "-nst")) ||
-            (!strcmp(argv[arg], "--next-soft-thread")) ||
-            (!strcmp(argv[arg], "-nht")) ||
-            (!strcmp(argv[arg], "--next-hard-thread"))
-        )
+        break;
+
+        case FCS_OPT_NEXT_SOFT_THREAD: /* STRINGS=-nst|--next-soft-thread; */
+        case FCS_OPT_NEXT_HARD_THREAD: /* STRINGS=-nht|--next-hard-thread; */
         {
             int ret;
-            int is_st = ((!strcmp(argv[arg], "-nst")) || (!strcmp(argv[arg], "--next-soft-thread")));
 
             ret =
-                is_st ?
-                    freecell_solver_user_next_soft_thread(instance) :
-                    freecell_solver_user_next_hard_thread(instance)
+                (opt == FCS_OPT_NEXT_SOFT_THREAD)
+                    ? freecell_solver_user_next_soft_thread(instance)
+                    : freecell_solver_user_next_hard_thread(instance)
                     ;
 
             if (ret)
@@ -601,71 +1178,64 @@ int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
 
                 *error_string = errstr;
 
-                *last_arg = arg;
-
-                return FCS_CMD_LINE_ERROR_IN_ARG;
+                RET_ERROR_IN_ARG() ;
             }
         }
-        else if ((!strcmp(argv[arg], "-step")) || (!strcmp(argv[arg], "--soft-thread-step")))
+        break;
+
+        case FCS_OPT_SOFT_THREAD_STEP: /* STRINGS=-step|--soft-thread-step; */
         {
-            arg++;
-            if (arg == argc)
-            {
-                *last_arg = arg-1;
-                return FCS_CMD_LINE_PARAM_WITH_NO_ARG;
-            }
+            PROCESS_OPT_ARG() ;
 
             freecell_solver_user_set_soft_thread_step(
                 instance,
-                atoi(argv[arg])
+                atoi((*arg))
             );
         }
-        else if ((!strcmp(argv[arg], "--reparent-states")))
+        break;
+
+        case FCS_OPT_REPARENT_STATES: /* STRINGS=--reparent-states; */
         {
             freecell_solver_user_set_reparent_states(
                 instance,
                 1
                 );
         }
-        else if ((!strcmp(argv[arg], "--calc-real-depth")))
+        break;
+
+        case FCS_OPT_CALC_REAL_DEPTH: /* STRINGS=--calc-real-depth; */
         {
             freecell_solver_user_set_calc_real_depth(
                 instance,
                 1);
         }
-        else if ((!strcmp(argv[arg], "--st-name")))
+        break;
+
+        case FCS_OPT_ST_NAME: /* STRINGS=--st-name; */
         {
-            arg++;
-            if (arg == argc)
-            {
-                *last_arg = arg-1;
-                return FCS_CMD_LINE_PARAM_WITH_NO_ARG;
-            }
-            freecell_solver_user_set_soft_thread_name(instance, argv[arg]);
+            PROCESS_OPT_ARG() ;
+
+            freecell_solver_user_set_soft_thread_name(instance, (*arg));
         }
-        else if ((!strcmp(argv[arg], "--prelude")))
+        break;
+
+        case FCS_OPT_PRELUDE: /* STRINGS=--prelude; */
         {
-            arg++;
-            if (arg == argc)
-            {
-                *last_arg = arg-1;
-                return FCS_CMD_LINE_PARAM_WITH_NO_ARG;
-            }
-            freecell_solver_user_set_hard_thread_prelude(instance, argv[arg]);
+            PROCESS_OPT_ARG() ;
+
+            freecell_solver_user_set_hard_thread_prelude(instance, (*arg));
         }
-        else if ((!strcmp(argv[arg], "-opt-to")) || (!strcmp(argv[arg], "--optimization-tests-order")))
+        break;
+
+        case FCS_OPT_OPTIMIZATION_TESTS_ORDER: /* STRINGS=-opt-to|--optimization-tests-order; */
         {
             char * fcs_user_errstr;
-            arg++;
-            if (arg == argc)
-            {
-                *last_arg = arg-1;
-                return FCS_CMD_LINE_PARAM_WITH_NO_ARG;
-            }
+
+            PROCESS_OPT_ARG() ;
 
             ret = freecell_solver_user_set_optimization_scan_tests_order(
                     instance,
-                    argv[arg],
+                    (*arg),
                     &fcs_user_errstr
                     );
 
@@ -681,26 +1251,22 @@ int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
 
                 *error_string = errstr;
 
-                *last_arg = arg;
-                return FCS_CMD_LINE_ERROR_IN_ARG;
+                RET_ERROR_IN_ARG() ;
             }
         }
-        else if ((!strcmp(argv[arg], "--scans-synergy")))
+        break;
+
+        case FCS_OPT_SCANS_SYNERGY: /* STRINGS=--scans-synergy; */
         {
             int value;
 
-            arg++;
-            if (arg == argc)
-            {
-                *last_arg = arg-1;
-                return FCS_CMD_LINE_PARAM_WITH_NO_ARG;
-            }
+            PROCESS_OPT_ARG() ;
 
-            if (!strcmp(argv[arg], "none"))
+            if (!strcmp((*arg), "none"))
             {
                 value = 0;
             }
-            else if (!strcmp(argv[arg], "dead-end-marks"))
+            else if (!strcmp((*arg), "dead-end-marks"))
             {
                 value = 1;
             }
@@ -708,12 +1274,12 @@ int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
             {
                 char * errstr;
 
-                errstr = malloc(strlen(argv[arg])+500);
+                errstr = malloc(strlen((*arg))+500);
 
-                sprintf(errstr, "Unknown scans' synergy type \"%s\"!\n", argv[arg]);
-                *last_arg = arg;
+                sprintf(errstr, "Unknown scans' synergy type \"%s\"!\n", (*arg));
                 *error_string = errstr;
-                return FCS_CMD_LINE_ERROR_IN_ARG;
+
+                RET_ERROR_IN_ARG() ;
             }
 
             freecell_solver_user_set_scans_synergy(
@@ -721,23 +1287,18 @@ int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
                 value
                 );
         }
-        else if ((!strcmp(argv[arg], "-ni")) ||
-                 (!strcmp(argv[arg], "--next-instance")))
-        {
-            freecell_solver_user_next_instance(instance);
-        }
-        else if (!strcmp(argv[arg], "--reset"))
+        break;
+
+        case FCS_OPT_RESET: /* STRINGS=--reset; */
         {
             freecell_solver_user_reset(instance);
         }
-        else if (!strcmp(argv[arg], "--read-from-file"))
+        break;
+
+        case FCS_OPT_READ_FROM_FILE: /* STRINGS=--read-from-file; */
         {
-            arg++;
-            if (arg == argc)
-            {
-                *last_arg = arg-1;
-                return FCS_CMD_LINE_PARAM_WITH_NO_ARG;
-            }
+            PROCESS_OPT_ARG() ;
+
             if (file_nesting_count == 0)
             {
                 /* do nothing */
@@ -752,14 +1313,14 @@ int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
                 size_t num_read;
                 args_man_t * args_man;
 
-                s = argv[arg];
+                s = (*arg);
                 while(isdigit(*s))
                 {
                     s++;
                 }
                 if (*s == ',')
                 {
-                    num_to_skip = atoi(argv[arg]);
+                    num_to_skip = atoi((*arg));
                     s++;
                 }
 
@@ -774,18 +1335,18 @@ int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
                 }
                 else
                 {
-                    /* 
+                    /*
                      * Initialize f to NULL so it will be initialized
                      * */
                     f = NULL;
                 }
 
-                /* Try to open from the local path */                
+                /* Try to open from the local path */
                 if (f == NULL)
                 {
                     f = fopen(s, "rt");
                 }
-                
+
                 /* If we still could not open it return an error */
                 if (f == NULL)
                 {
@@ -797,9 +1358,8 @@ int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
                             s);
 
                     *error_string = err_str;
-                    *last_arg = arg;
 
-                    return FCS_CMD_LINE_ERROR_IN_ARG;
+                    RET_ERROR_IN_ARG() ;
                 }
                 fseek(f, 0, SEEK_END);
                 file_len = ftell(f);
@@ -809,27 +1369,23 @@ int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
                     *error_string = strdup("Could not allocate enough memory to parse the file. Quitting.\n");
                     fclose(f);
 
-                    *last_arg = arg;
-
-                    return FCS_CMD_LINE_ERROR_IN_ARG;
+                    RET_ERROR_IN_ARG() ;
                 }
                 fseek(f,0,SEEK_SET);
                 num_read = fread(buffer, 1, file_len, f);
                 fclose(f);
                 buffer[num_read] = '\0';
 
-                args_man = freecell_solver_args_man_alloc();
-                ret = freecell_solver_args_man_chop(args_man, buffer);
+                args_man = fc_solve_args_man_alloc();
+                ret = fc_solve_args_man_chop(args_man, buffer);
                 free(buffer);
                 if (ret != 0)
                 {
                     *error_string =
                         strdup("Could not parse the file. Quitting\n");
-                    freecell_solver_args_man_free(args_man);
+                    fc_solve_args_man_free(args_man);
 
-                    *last_arg = arg;
-
-                    return FCS_CMD_LINE_ERROR_IN_ARG;
+                    RET_ERROR_IN_ARG() ;
                 }
 
                 if (num_to_skip >= args_man->argc)
@@ -858,37 +1414,33 @@ int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
                     }
                     else if (ret != FCS_CMD_LINE_OK)
                     {
-                        freecell_solver_args_man_free(args_man);
+                        fc_solve_args_man_free(args_man);
                         return ret;
                     }
                 }
-                freecell_solver_args_man_free(args_man);
+                fc_solve_args_man_free(args_man);
             }
         }
-        else if ((!strcmp(argv[arg], "-l")) || (!strcmp(argv[arg], "--load-config")))
+        break;
+
+        case FCS_OPT_LOAD_CONFIG: /* STRINGS=-l|--load-config; */
         {
-            arg++;
-            if (arg == argc)
-            {
-                *last_arg = arg-1;
-                return FCS_CMD_LINE_PARAM_WITH_NO_ARG;
-            }
+            PROCESS_OPT_ARG() ;
+
             {
                 int status;
                 args_man_t * preset_args;
                 char * dir = NULL;
 
-                status = read_preset(argv[arg], &preset_args, &dir, NULL);
+                status = read_preset((*arg), &preset_args, &dir, NULL);
                 if (status != 0)
                 {
                     char * err_str;
-                    err_str = malloc(strlen(argv[arg]) + 100);
-                    sprintf(err_str, "Unable to load the \"%s\" configuration!\n", argv[arg]);
+                    err_str = malloc(strlen((*arg)) + 100);
+                    sprintf(err_str, "Unable to load the \"%s\" configuration!\n", (*arg));
                     *error_string = err_str;
 
-                    *last_arg = arg;
-
-                    return FCS_CMD_LINE_ERROR_IN_ARG;
+                    RET_ERROR_IN_ARG() ;
                 }
                 else
                 {
@@ -910,7 +1462,7 @@ int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
                     {
                         free(dir);
                     }
-                    freecell_solver_args_man_free(preset_args);
+                    fc_solve_args_man_free(preset_args);
 
                     if (ret == FCS_CMD_LINE_UNRECOGNIZED_OPTION)
                     {
@@ -923,14 +1475,14 @@ int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
                 }
             }
         }
-        else
-        {
-            *last_arg = arg;
-            return FCS_CMD_LINE_UNRECOGNIZED_OPTION;
+        break;
+
+
         }
+        /* OPT-PARSE-END */
     }
 
-    *last_arg = arg;
+    *last_arg = arg-&(argv[0]);
     return FCS_CMD_LINE_OK;
 }
 
