@@ -1,14 +1,14 @@
 /*
- * fcs.h - header file of freecell_solver_instance and of user-level
+ * fcs.h - header file of fc_solve_instance and of user-level
  * functions for Freecell Solver
  *
- * Written by Shlomi Fish (shlomif@vipe.technion.ac.il), 2000
+ * Written by Shlomi Fish ( http://www.shlomifish.org/ ), 2000
  *
  * This file is in the public domain (it's uncopyrighted).
  */
 
-#ifndef __FCS_H
-#define __FCS_H
+#ifndef FC_SOLVE__FCS_H
+#define FC_SOLVE__FCS_H
 
 #ifdef __cplusplus
 extern "C" {
@@ -48,6 +48,12 @@ extern "C" {
 
 #endif
 
+#if (FCS_STATE_STORAGE == FCS_STATE_STORAGE_JUDY) || (defined(INDIRECT_STACK_STATES) && (FCS_STACK_STORAGE == FCS_STACK_STORAGE_JUDY))
+
+#include <Judy.h>
+
+#endif
+
 
 #if (FCS_STATE_STORAGE == FCS_STATE_STORAGE_INTERNAL_HASH)
 
@@ -76,7 +82,7 @@ extern "C" {
  * */
 struct fcs_states_linked_list_item_struct
 {
-    fcs_state_with_locations_t * s;
+    fcs_state_extra_info_t * s;
     struct fcs_states_linked_list_item_struct * next;
 };
 
@@ -108,12 +114,12 @@ enum FCS_TESTS_ORDER_FLAGS
 
 /*
  * Declare these structures because they will be used within
- * freecell_solver_instance, and they will contain a pointer to it.
+ * fc_solve_instance, and they will contain a pointer to it.
  * */
-struct freecell_solver_hard_thread_struct;
-struct freecell_solver_soft_thread_struct;
+struct fc_solve_hard_thread_struct;
+struct fc_solve_soft_thread_struct;
 
-typedef struct freecell_solver_hard_thread_struct freecell_solver_hard_thread_t;
+typedef struct fc_solve_hard_thread_struct fc_solve_hard_thread_t;
 
 struct fcs_tests_order_struct
 {
@@ -124,7 +130,7 @@ struct fcs_tests_order_struct
 
 typedef struct fcs_tests_order_struct fcs_tests_order_t;
 
-typedef struct freecell_solver_instance
+typedef struct fc_solve_instance
 {
 #if (FCS_STATE_STORAGE == FCS_STATE_STORAGE_INDIRECT)
     /* The sort-margin */
@@ -189,7 +195,7 @@ typedef struct freecell_solver_instance
         int iter_num,
         int depth,
         void * instance,
-        fcs_state_with_locations_t * state,
+        fcs_state_extra_info_t * state_val,
         int parent_iter_num
         );
     void * debug_iter_output_context;
@@ -201,6 +207,8 @@ typedef struct freecell_solver_instance
      * */
 #if (FCS_STATE_STORAGE == FCS_STATE_STORAGE_LIBREDBLACK_TREE)
     struct rbtree * tree;
+#elif (FCS_STATE_STORAGE == FCS_STATE_STORAGE_JUDY)
+    Pvoid_t judy_array;
 #elif (FCS_STATE_STORAGE == FCS_STATE_STORAGE_LIBAVL_AVL_TREE)
     avl_tree * tree;
 #elif (FCS_STATE_STORAGE == FCS_STATE_STORAGE_LIBAVL_REDBLACK_TREE)
@@ -223,7 +231,7 @@ typedef struct freecell_solver_instance
     /*
      * The storage mechanism for the stacks assuming INDIRECT_STACK_STATES is
      * used.
-     * */    
+     * */
 #if (FCS_STACK_STORAGE == FCS_STACK_STORAGE_INTERNAL_HASH)
     SFO_hash_t * stacks_hash;
 #elif (FCS_STACK_STORAGE == FCS_STACK_STORAGE_LIBAVL_AVL_TREE)
@@ -236,6 +244,10 @@ typedef struct freecell_solver_instance
     GTree * stacks_tree;
 #elif (FCS_STACK_STORAGE == FCS_STACK_STORAGE_GLIB_HASH)
     GHashTable * stacks_hash;
+#elif (FCS_STACK_STORAGE == FCS_STACK_STORAGE_JUDY)
+    Pvoid_t stacks_judy_array;
+#else
+#error FCS_STACK_STORAGE is not set to a good value.
 #endif
 #endif
 
@@ -254,11 +266,35 @@ typedef struct freecell_solver_instance
      * the beginning of the execution of the algorithm. However, there
      * is a maximal limit to them which is set in config.h.
      *
-     * decks_num can be 4 or 8
+     * decks_num can be 1 or 2 .
      * */
+
+#ifndef HARD_CODED_NUM_FREECELLS
     int freecells_num;
+#define INSTANCE_FREECELLS_NUM (instance->freecells_num)
+#define LOCAL_FREECELLS_NUM (freecells_num)
+#else
+#define INSTANCE_FREECELLS_NUM HARD_CODED_NUM_FREECELLS
+#define LOCAL_FREECELLS_NUM HARD_CODED_NUM_FREECELLS
+#endif
+
+#ifndef HARD_CODED_NUM_STACKS
     int stacks_num;
+#define INSTANCE_STACKS_NUM (instance->stacks_num)
+#define LOCAL_STACKS_NUM (stacks_num)
+#else
+#define INSTANCE_STACKS_NUM HARD_CODED_NUM_STACKS
+#define LOCAL_STACKS_NUM HARD_CODED_NUM_STACKS
+#endif
+
+#ifndef HARD_CODED_NUM_DECKS
     int decks_num;
+#define INSTANCE_DECKS_NUM (instance->decks_num)
+#define LOCAL_DECKS_NUM (decks_num)
+#else
+#define INSTANCE_DECKS_NUM HARD_CODED_NUM_DECKS
+#define LOCAL_DECKS_NUM HARD_CODED_NUM_DECKS
+#endif
 
     /* What two adjacent cards in the same sequence can be: */
     int sequences_are_built_by;
@@ -296,12 +332,12 @@ typedef struct freecell_solver_instance
     int optimize_solution_path;
 
     /* This is a place-holder for the initial state */
-    fcs_state_with_locations_t * state_copy_ptr;
+    fcs_state_extra_info_t * state_copy_ptr_val;
 
     /* This is the final state that the scan recommends to the
      * interface
      * */
-    fcs_state_with_locations_t * final_state;
+    fcs_state_extra_info_t * final_state_val;
 
     /*
      * This is the number of states in the state collection.
@@ -316,7 +352,7 @@ typedef struct freecell_solver_instance
     int max_num_states_in_collection;
 
     int num_hard_threads;
-    struct freecell_solver_hard_thread_struct * * hard_threads;
+    struct fc_solve_hard_thread_struct * * hard_threads;
 
     /*
      * The next ID to allocate for a soft-thread.
@@ -338,7 +374,7 @@ typedef struct freecell_solver_instance
     /*
      * This is the hard-thread used for the optimization scan.
      * */
-    struct freecell_solver_hard_thread_struct * optimization_thread;
+    struct fc_solve_hard_thread_struct * optimization_thread;
 
     /*
      * A counter that determines how many of the hard threads that belong
@@ -357,7 +393,7 @@ typedef struct freecell_solver_instance
      * The tests order for the optimization scan as specified by the user.
      * */
     int opt_tests_order_set;
-    
+
     fcs_tests_order_t opt_tests_order;
 
     /*
@@ -366,18 +402,18 @@ typedef struct freecell_solver_instance
      * */
     int to_reparent_states;
 
-    /* 
+    /*
      * This variable determines how the scans cooperate with each other.
-     * 
-     * A value of 0 indicates that they don't and only share the same 
+     *
+     * A value of 0 indicates that they don't and only share the same
      * states collection.
-     * 
+     *
      * A value of 1 indicates that they mark states as dead-end,
      * which may help or hinder other scans.
      * */
     int scans_synergy;
 
-} freecell_solver_instance_t;
+} fc_solve_instance_t;
 
 
 
@@ -394,12 +430,12 @@ struct fcs_prelude_item_struct
 typedef struct fcs_prelude_item_struct fcs_prelude_item_t;
 
 
-struct freecell_solver_hard_thread_struct
+struct fc_solve_hard_thread_struct
 {
-    freecell_solver_instance_t * instance;
+    fc_solve_instance_t * instance;
 
     int num_soft_threads;
-    struct freecell_solver_soft_thread_struct * * soft_threads;
+    struct fc_solve_soft_thread_struct * * soft_threads;
 
     /*
      * The State Packs variables are used by all the state cache
@@ -410,7 +446,7 @@ struct freecell_solver_hard_thread_struct
      * Such allocation is possible, because at the worst situation
      * the last state is released.
      * */
-    fcs_state_with_locations_t * * state_packs;
+    fcs_state_keyval_pair_t * * state_packs;
     int max_num_state_packs;
     int num_state_packs;
     int num_states_in_last_pack;
@@ -438,7 +474,7 @@ struct freecell_solver_hard_thread_struct
     int num_times_step;
 
     /*
-     * This is the number of iterations that still have to be done for 
+     * This is the number of iterations that still have to be done for
      * soft_threads[st_idx]. It is reset to (st_idx+1)->num_times_step
      * when st_idx is incremented.
      * */
@@ -474,15 +510,15 @@ struct freecell_solver_hard_thread_struct
     fcs_compact_allocator_t * stacks_allocator;
 #endif
 
-    /* 
+    /*
      * This is a compact memory allocator for the move stacks associated
      * with the states in the states collection.
      * */
     fcs_compact_allocator_t * move_stacks_allocator;
 
-    /* 
-     * This is a move stack that is used and re-used by the 
-     * tests functions of this hard thread 
+    /*
+     * This is a move stack that is used and re-used by the
+     * tests functions of this hard thread
      * */
     fcs_move_stack_t * reusable_move_stack;
 
@@ -501,7 +537,7 @@ struct freecell_solver_hard_thread_struct
     int prelude_num_items;
     int prelude_idx;
     fcs_prelude_item_t * prelude;
-    
+
 };
 
 
@@ -519,21 +555,21 @@ struct freecell_solver_hard_thread_struct
 
 struct fcs_soft_dfs_stack_item_struct
 {
-    fcs_state_with_locations_t * state;
+    fcs_state_extra_info_t * state_val;
     fcs_derived_states_list_t derived_states_list;
     int current_state_index;
     int test_index;
     int num_freestacks;
     int num_freecells;
     int derived_states_random_indexes_max_size;
-    int * derived_states_random_indexes;  
+    int * derived_states_random_indexes;
 };
 
 typedef struct fcs_soft_dfs_stack_item_struct fcs_soft_dfs_stack_item_t;
 
-struct freecell_solver_soft_thread_struct
+struct fc_solve_soft_thread_struct
 {
-    freecell_solver_hard_thread_t * hard_thread;
+    fc_solve_hard_thread_t * hard_thread;
 
     /*
      * The ID of the soft thread inside the instance.
@@ -596,7 +632,7 @@ struct freecell_solver_soft_thread_struct
      * The first state to be checked by the scan. It is a kind of bootstrap
      * for the algorithm.
      * */
-    fcs_state_with_locations_t * first_state_to_check;
+    fcs_state_extra_info_t * first_state_to_check_val;
 
     /*
      * These are stacks used by the Soft-DFS for various uses.
@@ -626,7 +662,7 @@ struct freecell_solver_soft_thread_struct
      * num_freecells[i] - ditto for the freecells.
      *
      * */
-    
+
     fcs_soft_dfs_stack_item_t * soft_dfs_info;
 
     /* The depth of the DFS stacks */
@@ -641,7 +677,7 @@ struct freecell_solver_soft_thread_struct
      * The initial seed of this random number generator
      * */
     int rand_seed;
-    
+
 
     /*
      * A flag that indicates if this soft thread have already been
@@ -672,7 +708,7 @@ struct freecell_solver_soft_thread_struct
     char * name;
 };
 
-typedef struct freecell_solver_soft_thread_struct freecell_solver_soft_thread_t;
+typedef struct fc_solve_soft_thread_struct fc_solve_soft_thread_t;
 
 
 #define FCS_SOFT_DFS_STATES_TO_CHECK_GROW_BY 32
@@ -686,112 +722,71 @@ typedef struct freecell_solver_soft_thread_struct freecell_solver_soft_thread_t;
 #define FCS_A_STAR_WEIGHT_SEQS_OVER_RENEGADE_CARDS 3
 #define FCS_A_STAR_WEIGHT_DEPTH 4
 
-freecell_solver_instance_t * freecell_solver_alloc_instance(void);
+fc_solve_instance_t * fc_solve_alloc_instance(void);
 
-extern void freecell_solver_init_instance(
-    freecell_solver_instance_t * instance
+extern void fc_solve_init_instance(
+    fc_solve_instance_t * instance
     );
 
-extern void freecell_solver_free_instance(
-    freecell_solver_instance_t * instance
+extern void fc_solve_free_instance(
+    fc_solve_instance_t * instance
     );
 
-extern void freecell_solver_finish_instance(
-    freecell_solver_instance_t * instance
+extern void fc_solve_finish_instance(
+    fc_solve_instance_t * instance
     );
 
-extern int freecell_solver_solve_instance(
-    freecell_solver_instance_t * instance,
-    fcs_state_with_locations_t * init_state
+extern int fc_solve_solve_instance(
+    fc_solve_instance_t * instance,
+    fcs_state_extra_info_t * init_state_val
     );
 
-extern int freecell_solver_resume_instance(
-    freecell_solver_instance_t * instance
+extern int fc_solve_resume_instance(
+    fc_solve_instance_t * instance
     );
 
-extern void freecell_solver_unresume_instance(
-    freecell_solver_instance_t * instance
+extern void fc_solve_unresume_instance(
+    fc_solve_instance_t * instance
     );
 
-extern freecell_solver_soft_thread_t * freecell_solver_instance_get_soft_thread(
-    freecell_solver_instance_t * instance,
+extern fc_solve_soft_thread_t * fc_solve_instance_get_soft_thread(
+    fc_solve_instance_t * instance,
     int ht_idx,
     int st_idx
     );
 
-extern freecell_solver_soft_thread_t * freecell_solver_new_soft_thread(
-    freecell_solver_soft_thread_t * soft_thread
+extern fc_solve_soft_thread_t * fc_solve_new_soft_thread(
+    fc_solve_soft_thread_t * soft_thread
     );
 
-extern freecell_solver_soft_thread_t * freecell_solver_new_hard_thread(
-    freecell_solver_instance_t * instance
+extern fc_solve_soft_thread_t * fc_solve_new_hard_thread(
+    fc_solve_instance_t * instance
     );
 
-extern int freecell_solver_hard_dfs_solve_for_state(
-    freecell_solver_soft_thread_t * soft_thread,
-    fcs_state_with_locations_t * ptr_state_with_locations,
-    int depth,
-    int ignore_osins
+int fc_solve_a_star_or_bfs_do_solve(
+    fc_solve_soft_thread_t * soft_thread
     );
 
-extern int freecell_solver_soft_dfs_solve(
-    freecell_solver_soft_thread_t * soft_thread,
-    fcs_state_with_locations_t * ptr_state_with_locations_orig
+extern void fc_solve_soft_thread_init_soft_dfs(
+    fc_solve_soft_thread_t * soft_thread
     );
 
-extern int freecell_solver_random_dfs_solve(
-    freecell_solver_soft_thread_t * soft_thread,
-    fcs_state_with_locations_t * ptr_state_with_locations_orig
+extern void fc_solve_soft_thread_init_a_star_or_bfs(
+    fc_solve_soft_thread_t * soft_thread
     );
 
 
-extern void freecell_solver_a_star_initialize_rater(
-    freecell_solver_soft_thread_t * soft_thread,
-    fcs_state_with_locations_t * ptr_state_with_locations
-    );
-
-extern int freecell_solver_a_star_or_bfs_do_solve_or_resume(
-    freecell_solver_soft_thread_t * soft_thread,
-    fcs_state_with_locations_t * ptr_state_with_locations_orig,
-    int resume
-    );
-
-extern int freecell_solver_hard_dfs_resume_solution(
-    freecell_solver_soft_thread_t * soft_thread,
-    int depth
-    );
-
-extern int freecell_solver_soft_dfs_resume_solution(
-    freecell_solver_soft_thread_t * soft_thread
-    );
-
-extern int freecell_solver_random_dfs_resume_solution(
-    freecell_solver_soft_thread_t * soft_thread
-    );
-
-
-extern int freecell_solver_a_star_or_bfs_solve(
-    freecell_solver_soft_thread_t * soft_thread,
-    fcs_state_with_locations_t * ptr_state_with_locations_orig
-    );
-
-extern int freecell_solver_a_star_or_bfs_resume_solution(
-    freecell_solver_soft_thread_t * soft_thread
-    );
-
-extern int freecell_solver_soft_dfs_or_random_dfs_do_solve_or_resume(
-    freecell_solver_soft_thread_t * soft_thread,
-    fcs_state_with_locations_t * ptr_state_with_locations_orig,
-    int resume,
+extern int fc_solve_soft_dfs_do_solve(
+    fc_solve_soft_thread_t * soft_thread,
     int to_randomize
     );
 
-extern void freecell_solver_recycle_instance(
-    freecell_solver_instance_t * instance
+extern void fc_solve_recycle_instance(
+    fc_solve_instance_t * instance
         );
 
 #ifdef __cplusplus
 }
 #endif
 
-#endif /* __FCS_H */
+#endif /* FC_SOLVE__FCS_H */
