@@ -1,6 +1,6 @@
 
-#ifndef __FCS_CL_H
-#define __FCS_CL_H
+#ifndef FC_SOLVE__FCS_CL_H
+#define FC_SOLVE__FCS_CL_H
 
 #include "fcs_user.h"
 
@@ -62,4 +62,4 @@ extern int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
 };
 #endif
 
-#endif /* #ifndef __FCS_CL_H */
+#endif /* #ifndef FC_SOLVE__FCS_CL_H */
