@@ -2,7 +2,7 @@
 
 use strict;
 
-my %presets = 
+my %presets =
 (
     'bakers_game' => [ qw(i freecell sbb suit) ],
     'bakers_dozen' => [ qw(s 13 f 0 d 1 sbb rank sm limited esf none to 0123456789 at 0123456789) ],
@@ -43,7 +43,7 @@ sub compile_preset
     while ($cmd = shift(@params))
     {
         $arg = shift(@params);
-        
+
         if ($cmd =~ /^(i|inherits?)$/)
         {
             if (!exists($presets{$arg}))
@@ -58,7 +58,7 @@ sub compile_preset
             {
                 die "Argument to stacks is not an integer!\n";
             }
-            $compiled->{'stacks'} = $arg;            
+            $compiled->{'stacks'} = $arg;
         }
         elsif ($cmd =~ /^(f|freecells?)$/)
         {
@@ -121,7 +121,7 @@ sub compile_preset
             die "Unknown Command $cmd\n";
         }
     }
-    
+
     };
 
     if ($@)
@@ -136,22 +136,22 @@ sub preset_to_string
 {
     my $preset_name = shift;
     my $pc = shift;
-    
+
     my @lines = ();
 
     my $arg;
 
     eval
     {
-    
+
     push @lines, ("FCS_PRESET_" . uc($preset_name));
-    
+
     if (!exists($pc->{'freecells'}))
     {
         die "Freecells were not defined!\n";
     }
     push @lines, $pc->{'freecells'};
-    
+
     if (!exists($pc->{'stacks'}))
     {
         die "Stacks were not defined!\n";
@@ -179,7 +179,7 @@ sub preset_to_string
 
     if (!exists($pc->{'empty_stacks_fill'}))
     {
-        die "Empty Stacks Fill is undefined!\n";        
+        die "Empty Stacks Fill is undefined!\n";
     }
     $arg = $pc->{'empty_stacks_fill'};
     push @lines, "FCS_ES_FILLED_BY_" . (($arg eq "none") ? "NONE" : ($arg eq "any_card") ? "ANY_CARD" : "KINGS_ONLY");
@@ -216,14 +216,14 @@ sub preset_to_docbook_string
     my @lines;
 
     push @lines, join(" ", (map { ucfirst($_) } split(/_/, $preset_name)));
-    
+
     push @lines, ($pc->{'stacks'}, $pc->{'freecells'}, $pc->{'decks'});
 
     my $sbb = $pc->{'seqs_build_by'};
-    push @lines, 
-        (($sbb eq "ac") ? "Alternate Colour" : 
-            ($sbb eq "suit") ? "Suit" : 
-            "Rank");    
+    push @lines,
+        (($sbb eq "ac") ? "Alternate Colour" :
+            ($sbb eq "suit") ? "Suit" :
+            "Rank");
 
     my $arg = $pc->{'empty_stacks_fill'};
     push @lines, (($arg eq "none") ? "None" : ($arg eq "any_card") ? "Any Card" : "Kings Only");
@@ -233,32 +233,121 @@ sub preset_to_docbook_string
     return join("", map { "    <entry>$_</entry>\n" } @lines);
 }
 
-my $docbook = 0;
 
-if ($ARGV[0] = "--docbook")
+sub preset_to_perl_module
 {
-    $docbook = 1;
+    my $preset_name = shift;
+    my $pc = shift;
+
+    my %sbb_map =
+    (
+        'ac' => "alt_color",
+        'suit' => "suit",
+        'rank' => "rank",
+    );
+
+    my $sbb = $sbb_map{$pc->{'seqs_build_by'}}
+        or die "Hoola";
+
+    my %esf_map =
+    (
+        'kings_only' => "kings",
+        'none' => "none",
+        'any_card' => "any",
+    );
+    my $esf = $esf_map{$pc->{'empty_stacks_fill'}}
+        or die "BlahBlajjor";
+
+    my $seq_move = $pc->{sequence_move} ? "unlimited" : "limited";
+
+    return <<"EOF";
+    "$preset_name" =>
+        Games::Solitaire::Verify::VariantParams->new(
+            {
+                'num_decks' => $pc->{decks},
+                'num_columns' => $pc->{stacks},
+                'num_freecells' => $pc->{freecells},
+                'sequence_move' => "$seq_move",
+                'seq_build_by' => "$sbb",
+                'empty_stacks_filled_by' => "$esf",
+            }
+        ),
+EOF
 }
 
+sub preset_to_pod
+{
+    my $preset_name = shift;
+    my $pc = shift;
+
+    return "=item * $preset_name\n\n";
+}
+
+my $mode = "c";
+
+my $mode_arg = shift(@ARGV);
+
+if ($mode_arg eq "--docbook")
+{
+    $mode = "docbook";
+}
+elsif ($mode_arg eq "--perlmod")
+{
+    $mode = "perl-mod";
+}
+elsif ($mode_arg eq "--pod")
+{
+    $mode = "pod";
+}
+
+my %mode_callbacks =
+(
+    "c" => \&preset_to_string,
+    "docbook" => \&preset_to_docbook_string,
+    "perl-mod" => \&preset_to_perl_module,
+    "pod" => \&preset_to_pod,
+);
+
 foreach my $preset_name (sort {$a cmp $b } keys(%presets))
 {
-    if ($docbook && ($preset_name eq "simple_simon"))
+    if (($mode ne "c") && ($preset_name eq "simple_simon"))
     {
         next;
     }
-        
+
     my $preset_compiled = compile_preset($preset_name);
-    push @strings, ($docbook ? \&preset_to_docbook_string : \&preset_to_string)->($preset_name, $preset_compiled);
+    push @strings, $mode_callbacks{$mode}->($preset_name, $preset_compiled);
 }
 
-if ($docbook)
+if ($mode eq "docbook")
 {
     print join("", map { "<row>\n$_</row>\n" } @strings);
 }
-else
+elsif ($mode eq "c")
 {
     print "static const fcs_preset_t fcs_presets[" . scalar(@strings) . "] = \n";
     print "{\n";
     print join("", @strings);
     print "};\n";
 }
+elsif ($mode eq "perl-mod")
+{
+    print "my %variants_map =\n";
+    print "(\n";
+    print join("", @strings);
+    print ");\n";
+}
+elsif ($mode eq "pod")
+{
+    print "=head1 PARAMETERS\n\n";
+
+    print "=head2 Variants IDs\n\n";
+
+    print "This is a list of the available variant IDS.\n\n";
+
+    print "=over 4\n\n";
+
+    print join("",@strings);
+
+    print "=back\n\n";
+}
