@@ -1,7 +1,7 @@
 /*
  * lib.c - library interface functions of Freecell Solver.
  *
- * Written by Shlomi Fish (shlomif@vipe.technion.ac.il), 2000
+ * Written by Shlomi Fish ( http://www.shlomifish.org/ ), 2000
  *
  * This file is in the public domain (it's uncopyrighted).
  */
@@ -20,7 +20,7 @@
 
 struct fcs_instance_item_struct
 {
-    freecell_solver_instance_t * instance;
+    fc_solve_instance_t * instance;
     int ret;
     int limit;
 };
@@ -29,17 +29,17 @@ typedef struct fcs_instance_item_struct fcs_instance_item_t;
 
 struct fcs_user_struct
 {
-    /* 
+    /*
      * This is a list of several consecutive instances that are run
      * one after the other in case the previous ones could not solve
-     * the board 
+     * the board
      * */
     fcs_instance_item_t * instances_list;
     int num_instances;
     int max_num_instances;
-    
+
     int current_instance_idx;
-    /* 
+    /*
      * The global (sequence-wide) limit of the iterations. Used
      * by limit_iterations() and friends
      * */
@@ -55,16 +55,16 @@ struct fcs_user_struct
     /*
      * A pointer to the currently active instance out of the sequence
      * */
-    freecell_solver_instance_t * instance;
-    fcs_state_with_locations_t state;
-    fcs_state_with_locations_t running_state;
+    fc_solve_instance_t * instance;
+    fcs_state_keyval_pair_t state;
+    fcs_state_keyval_pair_t running_state;
     int ret;
     int state_validity_ret;
     fcs_card_t state_validity_card;
     freecell_solver_user_iter_handler_t iter_handler;
     void * iter_handler_context;
 
-    freecell_solver_soft_thread_t * soft_thread;
+    fc_solve_soft_thread_t * soft_thread;
 
 #ifdef INDIRECT_STACK_STATES
     char indirect_stacks_buffer[MAX_NUM_STACKS << 7];
@@ -81,27 +81,27 @@ static void user_initialize(
         )
 {
     const fcs_preset_t * freecell_preset;
-    
-    freecell_solver_get_preset_by_name(
+
+    fc_solve_get_preset_by_name(
         "freecell",
         &freecell_preset
         );
 
     fcs_duplicate_preset(ret->common_preset, *freecell_preset);
-        
+
     ret->max_num_instances = 10;
     ret->instances_list = malloc(sizeof(ret->instances_list[0]) * ret->max_num_instances);
     ret->num_instances = 1;
     ret->current_instance_idx = 0;
-    ret->instance = freecell_solver_alloc_instance();
-    freecell_solver_apply_preset_by_ptr(ret->instance, &(ret->common_preset));
+    ret->instance = fc_solve_alloc_instance();
+    fc_solve_apply_preset_by_ptr(ret->instance, &(ret->common_preset));
     ret->instances_list[ret->current_instance_idx].instance = ret->instance;
     ret->instances_list[ret->current_instance_idx].ret = ret->ret = FCS_STATE_NOT_BEGAN_YET;
     ret->instances_list[ret->current_instance_idx].limit = -1;
     ret->current_iterations_limit = -1;
 
     ret->soft_thread =
-        freecell_solver_instance_get_soft_thread(
+        fc_solve_instance_get_soft_thread(
             ret->instance, 0,0
             );
 
@@ -131,8 +131,8 @@ int freecell_solver_user_apply_preset(
 
     user = (fcs_user_t*)user_instance;
 
-    status = 
-        freecell_solver_get_preset_by_name(
+    status =
+        fc_solve_get_preset_by_name(
             preset_name,
             &new_preset_ptr
             );
@@ -144,7 +144,7 @@ int freecell_solver_user_apply_preset(
 
     for(i = 0 ; i < user->num_instances ; i++)
     {
-        status = freecell_solver_apply_preset_by_ptr(
+        status = fc_solve_apply_preset_by_ptr(
             user->instances_list[i].instance,
             new_preset_ptr
             );
@@ -199,7 +199,7 @@ int freecell_solver_user_set_tests_order(
     user = (fcs_user_t*)user_instance;
 
     return
-        freecell_solver_apply_tests_order(
+        fc_solve_apply_tests_order(
             &(user->soft_thread->tests_order),
             tests_order,
             error_string
@@ -234,21 +234,21 @@ static void recycle_instance(
     }
     else if (user->instances_list[i].ret == FCS_STATE_SUSPEND_PROCESS)
     {
-        freecell_solver_unresume_instance(user->instances_list[i].instance);
+        fc_solve_unresume_instance(user->instances_list[i].instance);
     }
 
     if (user->instances_list[i].ret != FCS_STATE_NOT_BEGAN_YET)
     {
-        freecell_solver_recycle_instance(user->instances_list[i].instance);
+        fc_solve_recycle_instance(user->instances_list[i].instance);
         /*
-         * We have to initialize init_num_times to 0 here, because it may not 
+         * We have to initialize init_num_times to 0 here, because it may not
          * get initialized again, and now the num_times of the instance
          * is equal to 0.
          * */
         user->init_num_times = 0;
     }
 
-    user->instances_list[i].ret = FCS_STATE_NOT_BEGAN_YET;    
+    user->instances_list[i].ret = FCS_STATE_NOT_BEGAN_YET;
 }
 
 int freecell_solver_user_resume_solution(
@@ -261,27 +261,31 @@ int freecell_solver_user_resume_solution(
     fcs_user_t * user;
 
     user = (fcs_user_t*)user_instance;
-    
-    /* 
+
+    /*
      * I expect user->current_instance_idx to be initialized at some value.
      * */
-    for( ; 
+    for( ;
         run_for_first_iteration || ((user->current_instance_idx < user->num_instances) && (ret == FCS_STATE_IS_NOT_SOLVEABLE)) ;
         recycle_instance(user, user->current_instance_idx), user->current_instance_idx++
        )
     {
         run_for_first_iteration = 0;
-        
+
         user->instance = user->instances_list[user->current_instance_idx].instance;
 
         if (user->instances_list[user->current_instance_idx].ret == FCS_STATE_NOT_BEGAN_YET)
         {
+            int status;
+            fc_solve_instance_t * instance = user->instance;
 
-            user->state = freecell_solver_initial_user_state_to_c(
+            status = fc_solve_initial_user_state_to_c(
                 user->state_string_copy,
-                user->instance->freecells_num,
-                user->instance->stacks_num,
-                user->instance->decks_num
+                &(user->state.s),
+                &(user->state.info),
+                INSTANCE_FREECELLS_NUM,
+                INSTANCE_STACKS_NUM,
+                INSTANCE_DECKS_NUM
 #ifdef FCS_WITH_TALONS
                 ,user->instance->talon_type
 #endif
@@ -290,11 +294,18 @@ int freecell_solver_user_resume_solution(
 #endif
                 );
 
-            user->state_validity_ret = freecell_solver_check_state_validity(
-                &user->state,
-                user->instance->freecells_num,
-                user->instance->stacks_num,
-                user->instance->decks_num,
+            if (status != FCS_USER_STATE_TO_C__SUCCESS)
+            {
+                user->ret = FCS_STATE_INVALID_STATE;
+                user->state_validity_ret = FCS_STATE_VALIDITY__PREMATURE_END_OF_INPUT;
+                return user->ret;
+            }
+
+            user->state_validity_ret = fc_solve_check_state_validity(
+                &(user->state.info),
+                INSTANCE_FREECELLS_NUM,
+                INSTANCE_STACKS_NUM,
+                INSTANCE_DECKS_NUM,
 #ifdef FCS_WITH_TALONS
                 FCS_TALON_NONE,
 #endif
@@ -306,19 +317,24 @@ int freecell_solver_user_resume_solution(
                 return user->ret;
             }
 
-        
+
             /* running_state is a normalized state. So I'm duplicating
              * state to it before state is canonized
              * */
-            fcs_duplicate_state(user->running_state, user->state);
-
-            fcs_canonize_state(
-                &user->state,
-                user->instance->freecells_num,
-                user->instance->stacks_num
+            fcs_duplicate_state(
+                user->running_state.s,
+                user->running_state.info,
+                user->state.s,
+                user->state.info
                 );
 
-            freecell_solver_init_instance(user->instance);
+            fc_solve_canonize_state(
+                &user->state.info,
+                INSTANCE_FREECELLS_NUM,
+                INSTANCE_STACKS_NUM
+                );
+
+            fc_solve_init_instance(user->instance);
 
 #define global_limit() \
         (user->instance->num_times + user->current_iterations_limit - user->iterations_board_started_at)
@@ -361,40 +377,41 @@ int freecell_solver_user_resume_solution(
 
             user->init_num_times = init_num_times = user->instance->num_times;
 
-            ret = user->ret = 
-                user->instances_list[user->current_instance_idx].ret = 
-                freecell_solver_solve_instance(user->instance, &user->state);
+            ret = user->ret =
+                user->instances_list[user->current_instance_idx].ret =
+                fc_solve_solve_instance(user->instance, &user->state.info);
         }
         else
         {
 
             calc_max_iters();
-    
+
             user->init_num_times = init_num_times = user->instance->num_times;
-            
-            ret = user->ret = 
+
+            ret = user->ret =
                 user->instances_list[user->current_instance_idx].ret =
-                freecell_solver_resume_instance(user->instance);
+                fc_solve_resume_instance(user->instance);
         }
-        
+
         user->iterations_board_started_at += user->instance->num_times - init_num_times;
         user->init_num_times = user->instance->num_times;
 
         if (user->ret == FCS_STATE_WAS_SOLVED)
         {
-            freecell_solver_move_stack_normalize(
+            fc_solve_instance_t * instance = user->instance;
+            fc_solve_move_stack_normalize(
                 user->instance->solution_moves,
-                &(user->state),
-                user->instance->freecells_num,
-                user->instance->stacks_num,
-                user->instance->decks_num
+                &(user->state.info),
+                INSTANCE_FREECELLS_NUM,
+                INSTANCE_STACKS_NUM,
+                INSTANCE_DECKS_NUM
                 );
 
             break;
         }
         else if (user->ret == FCS_STATE_SUSPEND_PROCESS)
         {
-            /* 
+            /*
              * First - check if we exceeded our limit. If so - we must terminate
              * and return now.
              * */
@@ -403,8 +420,8 @@ int freecell_solver_user_resume_solution(
             {
                 break;
             }
-            
-            /* 
+
+            /*
              * Determine if we exceeded the instance-specific quota and if
              * so, designate it as unsolvable.
              * */
@@ -428,30 +445,34 @@ int freecell_solver_user_get_next_move(
     fcs_user_t * user;
 
     user = (fcs_user_t*)user_instance;
-    if (user->ret == FCS_STATE_WAS_SOLVED)
+
     {
-        int ret;
-
-        ret = fcs_move_stack_pop(
-            user->instance->solution_moves,
-            move
-            );
-
-        if (ret == 0)
+        fc_solve_instance_t * instance = user->instance;
+        if (user->ret == FCS_STATE_WAS_SOLVED)
         {
-            freecell_solver_apply_move(
-                &(user->running_state),
-                *move,
-                user->instance->freecells_num,
-                user->instance->stacks_num,
-                user->instance->decks_num
+            int ret;
+
+            ret = fcs_move_stack_pop(
+                user->instance->solution_moves,
+                move
                 );
+
+            if (ret == 0)
+            {
+                fc_solve_apply_move(
+                    &(user->running_state.info),
+                    *move,
+                    INSTANCE_FREECELLS_NUM,
+                    INSTANCE_STACKS_NUM,
+                    INSTANCE_DECKS_NUM
+                    );
+            }
+            return ret;
+        }
+        else
+        {
+            return 1;
         }
-        return ret;
-    }
-    else
-    {
-        return 1;
     }
 }
 
@@ -466,16 +487,20 @@ char * freecell_solver_user_current_state_as_string(
 
     user = (fcs_user_t *)user_instance;
 
-    return
-        freecell_solver_state_as_string(
-            &(user->running_state),
-            user->instance->freecells_num,
-            user->instance->stacks_num,
-            user->instance->decks_num,
-            parseable_output,
-            canonized_order_output,
-            display_10_as_t
-            );
+    {
+        fc_solve_instance_t * instance = user->instance;
+
+        return
+            fc_solve_state_as_string(
+                &(user->running_state.info),
+                INSTANCE_FREECELLS_NUM,
+                INSTANCE_STACKS_NUM,
+                INSTANCE_DECKS_NUM,
+                parseable_output,
+                canonized_order_output,
+                display_10_as_t
+                );
+    }
 }
 
 static void user_free_resources(
@@ -483,11 +508,11 @@ static void user_free_resources(
     )
 {
     int i;
-    
+
     for(i=0;i<user->num_instances;i++)
     {
         int ret_code = user->instances_list[i].ret;
-        
+
         if (ret_code == FCS_STATE_WAS_SOLVED)
         {
             fcs_move_stack_destroy(user->instance->solution_moves);
@@ -495,18 +520,18 @@ static void user_free_resources(
         }
         else if (ret_code == FCS_STATE_SUSPEND_PROCESS)
         {
-            freecell_solver_unresume_instance(user->instances_list[i].instance);
+            fc_solve_unresume_instance(user->instances_list[i].instance);
         }
 
         if (ret_code != FCS_STATE_NOT_BEGAN_YET)
         {
             if (ret_code != FCS_STATE_INVALID_STATE)
             {
-                freecell_solver_finish_instance(user->instances_list[i].instance);
+                fc_solve_finish_instance(user->instances_list[i].instance);
             }
         }
 
-        freecell_solver_free_instance(user->instances_list[i].instance);
+        fc_solve_free_instance(user->instances_list[i].instance);
     }
 
     free(user->instances_list);
@@ -562,23 +587,27 @@ void freecell_solver_user_set_solving_method(
     }        \
     user->common_preset.what = what;     \
     }
-    
+
 int freecell_solver_user_set_num_freecells(
     void * user_instance,
     int freecells_num
     )
 {
     fcs_user_t * user;
+#ifndef HARD_CODED_NUM_FREECELLS
     int i;
+#endif
 
     user = (fcs_user_t *)user_instance;
 
     if ((freecells_num < 0) || (freecells_num > MAX_NUM_FREECELLS))
-    {     
-        return 1;     
-    }    
+    {
+        return 1;
+    }
 
+#ifndef HARD_CODED_NUM_FREECELLS
     set_for_all_instances(freecells_num);
+#endif
 
     return 0;
 }
@@ -589,7 +618,9 @@ int freecell_solver_user_set_num_stacks(
     )
 {
     fcs_user_t * user;
+#ifndef HARD_CODED_NUM_STACKS
     int i;
+#endif
 
     user = (fcs_user_t *)user_instance;
 
@@ -597,7 +628,9 @@ int freecell_solver_user_set_num_stacks(
     {
         return 1;
     }
+#ifndef HARD_CODED_NUM_STACKS
     set_for_all_instances(stacks_num);
+#endif
 
     return 0;
 }
@@ -608,7 +641,9 @@ int freecell_solver_user_set_num_decks(
     )
 {
     fcs_user_t * user;
+#ifndef HARD_CODED_NUM_DECKS
     int i;
+#endif
 
     user = (fcs_user_t *)user_instance;
 
@@ -616,7 +651,9 @@ int freecell_solver_user_set_num_decks(
     {
         return 1;
     }
+#ifndef HARD_CODED_NUM_DECKS
     set_for_all_instances(decks_num);
+#endif
 
     return 0;
 }
@@ -710,7 +747,7 @@ char * freecell_solver_user_move_to_string(
     int standard_notation
     )
 {
-    return freecell_solver_move_to_string(move, standard_notation);
+    return fc_solve_move_to_string(move, standard_notation);
 }
 
 char * freecell_solver_user_move_to_string_w_state(
@@ -720,16 +757,18 @@ char * freecell_solver_user_move_to_string_w_state(
     )
 {
     fcs_user_t * user;
+    fc_solve_instance_t * instance;
 
     user = (fcs_user_t *)user_instance;
-    
-    return 
-        freecell_solver_move_to_string_w_state(
-            &(user->running_state), 
-            user->instance->freecells_num, 
-            user->instance->stacks_num, 
-            user->instance->decks_num, 
-            move, 
+    instance = user->instance;
+
+    return
+        fc_solve_move_to_string_w_state(
+            &(user->running_state.info),
+            INSTANCE_FREECELLS_NUM,
+            INSTANCE_STACKS_NUM,
+            INSTANCE_DECKS_NUM,
+            move,
             standard_notation
             );
 }
@@ -772,27 +811,32 @@ char * freecell_solver_user_get_invalid_state_error_string(
 
     user = (fcs_user_t *)user_instance;
 
-    if (user->state_validity_ret == 0)
+    if (user->state_validity_ret == FCS_STATE_VALIDITY__OK)
     {
         return strdup("");
     }
     fcs_card_perl2user(user->state_validity_card, card_str, print_ts);
 
-    if (user->state_validity_ret == 3)
+    if (user->state_validity_ret == FCS_STATE_VALIDITY__EMPTY_SLOT)
     {
         sprintf(string, "%s",
             "There's an empty slot in one of the stacks."
             );
     }
-    else
+    else if ((user->state_validity_ret == FCS_STATE_VALIDITY__EXTRA_CARD) ||
+           (user->state_validity_ret == FCS_STATE_VALIDITY__MISSING_CARD)
+          )
     {
         sprintf(string, "%s%s.",
-            ((user->state_validity_ret == 2)? "There's an extra card: " : "There's a missing card: "),
+            ((user->state_validity_ret == FCS_STATE_VALIDITY__EXTRA_CARD)? "There's an extra card: " : "There's a missing card: "),
             card_str
         );
     }
+    else if (user->state_validity_ret == FCS_STATE_VALIDITY__PREMATURE_END_OF_INPUT)
+    {
+        sprintf(string, "%s.", "Not enough input");
+    }
     return strdup(string);
-
 }
 
 int freecell_solver_user_set_sequences_are_built_by_type(
@@ -873,24 +917,28 @@ int freecell_solver_user_set_a_star_weight(
 
 }
 
-static void freecell_solver_user_iter_handler_wrapper(
+static void iter_handler_wrapper(
     void * user_instance,
     int iter_num,
     int depth,
     void * lp_instance,
-    fcs_state_with_locations_t * ptr_state_with_locations,
+    fcs_state_extra_info_t * ptr_state_val,
     int parent_iter_num
     )
 {
     fcs_user_t * user;
 
+    fcs_standalone_state_ptrs_t state;
+    state.key = ptr_state_val->key;
+    state.val = ptr_state_val;
+
     user = (fcs_user_t *)user_instance;
 
     user->iter_handler(
         user_instance,
         iter_num,
         depth,
-        (void *)ptr_state_with_locations,
+        (void *)&state,
         parent_iter_num,
         user->iter_handler_context
         );
@@ -899,79 +947,84 @@ static void freecell_solver_user_iter_handler_wrapper(
 }
 
 void freecell_solver_user_set_iter_handler(
-void * user_instance,
-freecell_solver_user_iter_handler_t iter_handler,
-void * iter_handler_context
-)
+    void * user_instance,
+    freecell_solver_user_iter_handler_t iter_handler,
+    void * iter_handler_context
+    )
 {
-fcs_user_t * user;
+    fcs_user_t * user;
 
-user = (fcs_user_t *)user_instance;
+    user = (fcs_user_t *)user_instance;
 
-if (iter_handler == NULL)
-{
-    user->instance->debug_iter_output = 0;
-}
-else
-{
-    /* Disable it temporarily while we change the settings */
-    user->instance->debug_iter_output = 0;
-    user->iter_handler = iter_handler;
-    user->iter_handler_context = iter_handler_context;
-    user->instance->debug_iter_output_context = user;
-    user->instance->debug_iter_output_func = freecell_solver_user_iter_handler_wrapper;
-    user->instance->debug_iter_output = 1;
-}
+    if (iter_handler == NULL)
+    {
+        user->instance->debug_iter_output = 0;
+    }
+    else
+    {
+        /* Disable it temporarily while we change the settings */
+        user->instance->debug_iter_output = 0;
+        user->iter_handler = iter_handler;
+        user->iter_handler_context = iter_handler_context;
+        user->instance->debug_iter_output_context = user;
+        user->instance->debug_iter_output_func = iter_handler_wrapper;
+        user->instance->debug_iter_output = 1;
+    }
 }
 
 char * freecell_solver_user_iter_state_as_string(
-void * user_instance,
-void * ptr_state,
-int parseable_output,
-int canonized_order_output,
-int display_10_as_t
+    void * user_instance,
+    void * ptr_state_void,
+    int parseable_output,
+    int canonized_order_output,
+    int display_10_as_t
 )
 {
-fcs_user_t * user;
+    fcs_user_t * user;
+    fc_solve_instance_t * instance;
+    fcs_standalone_state_ptrs_t * ptr_state;
 
-user = (fcs_user_t *)user_instance;
+    user = (fcs_user_t *)user_instance;
+    instance = user->instance;
 
-return
-    freecell_solver_state_as_string(
-        ptr_state,
-        user->instance->freecells_num,
-        user->instance->stacks_num,
-        user->instance->decks_num,
-        parseable_output,
-        canonized_order_output,
-        display_10_as_t
-        );
+    ptr_state = (fcs_standalone_state_ptrs_t *)ptr_state_void;
+
+    return
+        fc_solve_state_as_string(
+            ptr_state->val,
+            INSTANCE_FREECELLS_NUM,
+            INSTANCE_STACKS_NUM,
+            INSTANCE_DECKS_NUM,
+            parseable_output,
+            canonized_order_output,
+            display_10_as_t
+            );
 }
 
 void freecell_solver_user_set_random_seed(
-void * user_instance,
-int seed
+    void * user_instance,
+    int seed
 )
 {
-fcs_user_t * user;
+    fcs_user_t * user;
 
-user = (fcs_user_t *)user_instance;
+    user = (fcs_user_t *)user_instance;
 
-freecell_solver_rand_srand(user->soft_thread->rand_gen, (user->soft_thread->rand_seed = seed));
+    fc_solve_rand_srand(user->soft_thread->rand_gen, (user->soft_thread->rand_seed = seed));
 }
 
 int freecell_solver_user_get_num_states_in_collection(void * user_instance)
 {
-fcs_user_t * user;
+    fcs_user_t * user;
 
-user = (fcs_user_t *)user_instance;
+    user = (fcs_user_t *)user_instance;
 
-return user->instance->num_states_in_collection;
+    return user->instance->num_states_in_collection;
 }
 
 void freecell_solver_user_limit_num_states_in_collection(
-void * user_instance,
-int max_num_states
+    void * user_instance,
+    int max_num_states
     )
 {
     fcs_user_t * user;
@@ -986,11 +1039,11 @@ int freecell_solver_user_next_soft_thread(
     )
 {
     fcs_user_t * user;
-    freecell_solver_soft_thread_t * soft_thread;
+    fc_solve_soft_thread_t * soft_thread;
 
     user = (fcs_user_t *)user_instance;
 
-    soft_thread = freecell_solver_new_soft_thread(user->soft_thread);
+    soft_thread = fc_solve_new_soft_thread(user->soft_thread);
 
     if (soft_thread == NULL)
     {
@@ -1019,11 +1072,11 @@ int freecell_solver_user_next_hard_thread(
     )
 {
     fcs_user_t * user;
-    freecell_solver_soft_thread_t * soft_thread;
+    fc_solve_soft_thread_t * soft_thread;
 
     user = (fcs_user_t *)user_instance;
 
-    soft_thread = freecell_solver_new_hard_thread(user->instance);
+    soft_thread = fc_solve_new_hard_thread(user->instance);
 
     if (soft_thread == NULL)
     {
@@ -1071,7 +1124,7 @@ void freecell_solver_user_set_soft_thread_name(
     {
         free(user->soft_thread->name);
     }
-    user->soft_thread->name = strdup(name);    
+    user->soft_thread->name = strdup(name);
 }
 
 int freecell_solver_user_set_hard_thread_prelude(
@@ -1080,7 +1133,7 @@ int freecell_solver_user_set_hard_thread_prelude(
     )
 {
     fcs_user_t * user;
-    freecell_solver_hard_thread_t * hard_thread;
+    fc_solve_hard_thread_t * hard_thread;
 
     user = (fcs_user_t *)user_instance;
 
@@ -1137,12 +1190,12 @@ int freecell_solver_user_set_optimization_scan_tests_order(
 
     user->instance->opt_tests_order_set = 0;
 
-    ret = 
-        freecell_solver_apply_tests_order(
+    ret =
+        fc_solve_apply_tests_order(
             &(user->instance->opt_tests_order),
             tests_order,
             error_string
-            );    
+            );
 
     if (!ret)
     {
@@ -1183,35 +1236,35 @@ int freecell_solver_user_next_instance(
     fcs_user_t * user;
 
     user = (fcs_user_t *)user_instance;
-    
+
     user->num_instances++;
     if (user->num_instances == user->max_num_instances)
     {
         user->max_num_instances += 10;
-        user->instances_list = 
+        user->instances_list =
             realloc(
-                user->instances_list, 
+                user->instances_list,
                 sizeof(user->instances_list[0])*user->max_num_instances
                 );
     }
     user->current_instance_idx = user->num_instances-1;
-    user->instance = freecell_solver_alloc_instance();
+    user->instance = fc_solve_alloc_instance();
 
-    freecell_solver_apply_preset_by_ptr(user->instance, &(user->common_preset));
+    fc_solve_apply_preset_by_ptr(user->instance, &(user->common_preset));
 
-    /* 
+    /*
      * Switch the soft_thread variable so it won't refer to the old
      * instance
      * */
-    user->soft_thread = 
-        freecell_solver_instance_get_soft_thread(
+    user->soft_thread =
+        fc_solve_instance_get_soft_thread(
             user->instance, 0, 0
             );
 
     user->instances_list[user->current_instance_idx].instance = user->instance;
     user->instances_list[user->current_instance_idx].ret = user->ret = FCS_STATE_NOT_BEGAN_YET;
     user->instances_list[user->current_instance_idx].limit = -1;
-    
+
     return 0;
 }
 
@@ -1228,3 +1281,7 @@ int freecell_solver_user_reset(void * user_instance)
     return 0;
 }
 
+const char * freecell_solver_user_get_lib_version(void * user_instance)
+{
+    return VERSION;
+}
