@@ -5,7 +5,7 @@
 
 #include "fcs_cl.h"
 
-struct freecell_solver_display_information_context_struct
+struct fc_solve_display_information_context_struct
 {
     int debug_iter_state_output;
     int freecells_num;
@@ -21,7 +21,23 @@ struct freecell_solver_display_information_context_struct
     int standard_notation;
 };
 
-typedef struct freecell_solver_display_information_context_struct freecell_solver_display_information_context_t;
+typedef struct fc_solve_display_information_context_struct fc_solve_display_information_context_t;
+
+static void init_debug_context(
+    fc_solve_display_information_context_t * dc
+    )
+{
+    dc->debug_iter_state_output = 0;
+    dc->parseable_output = 0;
+    dc->canonized_order_output = 0;
+    dc->display_10_as_t = 0;
+    dc->display_parent_iter_num = 0;
+    dc->display_moves = 0;
+    dc->display_states = 1;
+    dc->standard_notation = 0;
+}
+
+
 
 static void my_iter_handler(
     void * user_instance,
@@ -32,8 +48,8 @@ static void my_iter_handler(
     void * lp_context
     )
 {
-    freecell_solver_display_information_context_t * context;
-    context = (freecell_solver_display_information_context_t*)lp_context;
+    fc_solve_display_information_context_t * context;
+    context = (fc_solve_display_information_context_t*)lp_context;
 
     fprintf(stdout, "Iteration: %i\n", iter_num);
     fprintf(stdout, "Depth: %i\n", depth);
@@ -98,35 +114,65 @@ static void my_iter_handler(
 #endif
 }
 
+struct help_screen_struct
+{
+    char * key;
+    char * screen;
+};
 
-static char * help_string =
+typedef struct help_screen_struct help_screen_t;
+
+help_screen_t help_screens[] = {
+{
+    "configs",
+"These configurations are usually faster than the unmodified run:\n"
+"\n"
+"    fc-solve -l cool-jives\n"
+"    fc-solve -l john-galt-line\n"
+"\n"
+"Or if you want an accurate verdict:\n"
+"\n"
+"    fc-solve -l fools-gold\n"
+"\n"
+"If you want to try constructing your own configurations refer to the\n"
+"USAGE file in the Freecell Solver distribution\n"
+},
+{
+    "options",
 "fc-solve [options] board_file\n"
 "\n"
 "If board_file is - or unspecified reads standard input\n"
 "\n"
 "Available Options:\n"
-"-h   --help           \n"
-"     display this help screen\n"
-"-i   --iter-output    \n"
+"-h     --help\n"
+"     display the default help screen\n"
+"--help-summary\n"
+"     display the summary help screen\n"
+"--version\n"
+"     display the version number of the components\n"
+"-i     --iter-output\n"
 "     display the iteration number and depth in every state that is checked\n"
-"-s   --state-output   \n"
+"-s     --state-output\n"
 "     also output the state in every state that is checked\n"
-"-p   --parseable-output \n"
+"-p     --parseable-output\n"
 "     Output the states in a format that is friendly to perl, grep and\n"
 "     friends.\n"
-"-c   --canonized-order-output \n"
+"-c     --canonized-order-output\n"
 "     Output the stacks and freecells according to their canonic order.\n"
 "     (That means that stacks and freecells won't retain their place.)\n"
-"-t   --display-10-as-t \n"
+"-t     --display-10-as-t\n"
 "     Display the card 10 as a capital T instead of \"10\".\n"
-"-m   --display-moves \n"
+"-m     --display-moves\n"
 "     Display the moves instead of the intermediate states.\n"
-"-sam      --display-states-and-moves \n"
+"-sam   --display-states-and-moves \n"
 "     Display both intermediate states and moves.\n"
-"-sn  --standard-notation \n"
+"-sn    --standard-notation\n"
 "     Display the moves in standard (non-verbose) notation.\n"
 "     (Applicable only if -m was specified)\n"
-"-pi  --display-parent-iter \n"
+"-snx   --standard-notation-extended\n"
+"     Display the moves in extended standard notation while specifying the\n"
+"     number of cards moved if applicable\n"
+"-pi    --display-parent-iter \n"
 "     Display the index of the parent iteration of each state in the\n"
 "     run-time dump.\n"
 "\n"
@@ -156,6 +202,7 @@ static char * help_string =
 "     der_katz           - Der Katzenschwanz\n"
 "     die_schlange       - Die Schlange\n"
 "     eight_off          - Eight Off\n"
+"     fan                - Fan\n"
 "     forecell           - Forecell\n"
 "     freecell           - Freecell\n"
 "     good_measure       - Good Measure\n"
@@ -189,7 +236,16 @@ static char * help_string =
 "        '8' - move cards to a different parent.\n"
 "        '9' - empty an entire stack into the freecells.\n"
 "\n"
-"     Simple Simon Tests:\n"
+"        Atomic Freecell Tests:\n"
+"\n"
+"        'A' - move a stack card to an empty stack.\n"
+"        'B' - move a stack card to a parent on a different stack.\n"
+"        'C' - move a stack card to a freecell.\n"
+"        'D' - move a freecel card to a parent.\n"
+"        'E' - move a freecel card to an empty stack.\n"
+"\n"
+"        Simple Simon Tests:\n"
+"\n"
 "        'a' - move a full sequence to the foundations.\n"
 "        'b' - move a sequence to a true parent of his.\n"
 "        'c' - move a whole stack sequence to a false parent (in order to\n"
@@ -239,9 +295,44 @@ static char * help_string =
 "     Move to the next Hard-Thread. This is a new group of scans to run\n"
 "     in their own system thread (assuming the executable was compiled with\n"
 "     support for them.)\n"
+"--st-name\n"
+"     Set the name of the soft-thread.\n"
+"\n"
+"--prelude [prelude_string]\n"
+"     Set the prelude string of the hard thread. A prelude is a static\n"
+"     sequence of iterations quotas that are executed at the beginning of\n"
+"     the search. The format is a list of [Limit]@[Soft-Thread Name]\n"
+"     delimited by commas.\n"
+"\n"
+"-ni         --next-instance\n"
+"     Move to the next distinct solver instance. This is a separate scan\n"
+"     which would run only if the previous ones returned an unsolvable\n"
+"     verdict.\n"
 "\n"
 "-opt         --optimize-solution\n"
 "     Try and optimize the solution for a small number of moves.\n"
+"-opt-to      --optimization-tests-order\n"
+"     The test order of the optimization scan.\n"
+"\n"
+"\n"
+"--reparent-states\n"
+"     Reparent states that have a larger depth than that of the state\n"
+"     from which they were reached a posteriori.\n"
+"--calc-real-depth\n"
+"     If --reparent-states is enabled, then explictly calculate the real\n"
+"     depth of a state by tracing its path to the initial state\n"
+"--scans-synergy {none|dead-end-marks}\n"
+"     Specifies the cooperation between the scans.\n"
+"\n"
+"\n"
+"--reset\n"
+"     Reset the program to its initial, unconfigured state.\n"
+"--read-from-file [{num_skip},]filename\n"
+"     Reads configuration parameter with the file while skipping num_skip\n"
+"     arguments from the beginning.\n"
+"-l [configuration]      --load-config [configuration]\n"
+"     Reads the configuration [configruration] and configures the solver\n"
+"     accordingly.\n"
 "\n"
 "\n"
 "Signals:\n"
@@ -251,8 +342,62 @@ static char * help_string =
 "\n"
 "\n"
 "Freecell Solver was written by Shlomi Fish.\n"
-"Homepage: http://vipe.technion.ac.il/~shlomif/freecell-solver/\n"
-"Send comments and suggestions to shlomif@vipe.technion.ac.il\n"
+"Homepage: http://fc-solve.berlios.de/\n"
+"Send comments and suggestions to http://www.shlomifish.org/me/contact-me/\n"
+},
+{
+    "real-help",
+"The environment variable FREECELL_SOLVER_DEFAULT_HELP sets the default help\n"
+"screen. The name of the help screen is the same name as its \"--help-\" flag\n"
+"but without the preceding \"--help-\". Type:\n"
+"\n"
+"    fc-solve --help-summary\n"
+"\n"
+"for the available help screens.\n"
+"\n"
+"Refer to your system's documentation for information on how to set environment\n"
+"variables.\n"
+},
+{
+    "problems",
+"To be discussed.\n"
+},
+{
+    "short-sol",
+"The following configurations may produce shorter solutions:\n"
+"\n"
+"    fc-solve -opt\n"
+"    fc-solve --method a-star -opt\n"
+"    fc-solve --reparent-states -opt\n"
+"    fc-solve --method a-star --reparent-states -opt\n"
+"\n"
+"If \"--method a-star\" is specified you can set the weights with\n"
+"-asw {comma separated list of 5 numeric weights}, which may improve\n"
+"the length of the solution. (refer to the USAGE file for more information)\n"
+},
+{
+    "summary",
+"fc-solve [flags] [board_file|-]\n"
+"\n"
+"Reads board from standard input by default or if a \"-\" is specified.\n"
+"\n"
+"- If it takes too long to finish, type \"fc-solve --help-configs\"\n"
+"- If it erroneously reports a board as unsolvable, try adding the\n"
+"  \"-to 01ABCDE\" flag\n"
+"- If the solution is too long type \"fc-solve --help-short-sol\"\n"
+"- To present the moves only try adding \"-m\" or \"-m -snx\"\n"
+"- For a description of all options type \"fc-solve --help-options\"\n"
+"- To deal with other problems type \"fc-solve --help-problems\"\n"
+"- To turn --help into something more useful, type\n"
+"  \"fc-solve --help-real-help\"\n"
+"\n"
+"Contact Shlomi Fish, http://www.shlomifish.org/ for more information.\n"
+},
+{
+    NULL,
+    NULL
+}
+}
 ;
 
 enum MY_FCS_CMD_LINE_RET_VALUES
@@ -261,6 +406,18 @@ enum MY_FCS_CMD_LINE_RET_VALUES
 
 };
 
+static void print_help_string(char * key)
+{
+    int i;
+    for(i=0;help_screens[i].key != NULL ; i++)
+    {
+        if (!strcmp(key, help_screens[i].key))
+        {
+            printf("%s", help_screens[i].screen);
+        }
+    }
+}
+
 static int cmd_line_callback(
     void * instance,
     int argc,
@@ -271,29 +428,56 @@ static int cmd_line_callback(
     void * context
     )
 {
-    freecell_solver_display_information_context_t * dc;
+    fc_solve_display_information_context_t * dc;
     *num_to_skip = 0;
 
-    dc = (freecell_solver_display_information_context_t * )context;
+    dc = (fc_solve_display_information_context_t * )context;
 
-    if ((!strcmp(argv[arg], "-h")) || (!strcmp(argv[arg], "--help")))
+    if (!strcmp(argv[arg], "--version"))
     {
-        printf("%s", help_string);
+        printf(
+            "fc-solve\nlibfreecell-solver version %s\n",
+            freecell_solver_user_get_lib_version(instance)
+        );
+        *ret = EXIT_AND_RETURN_0;
+        return FCS_CMD_LINE_STOP;
+    }
+    else if ((!strcmp(argv[arg], "-h")) || (!strcmp(argv[arg], "--help")))
+    {
+        char * help_key;
+
+        help_key = getenv("FREECELL_SOLVER_DEFAULT_HELP");
+        if (help_key == NULL)
+        {
+            help_key = "summary";
+        }
+        print_help_string(help_key);
+        *ret = EXIT_AND_RETURN_0;
+        return FCS_CMD_LINE_STOP;
+    }
+    else if (!strncmp(argv[arg], "--help-", 7))
+    {
+        print_help_string(argv[arg]+7);
         *ret = EXIT_AND_RETURN_0;
         return FCS_CMD_LINE_STOP;
     }
     else if ((!strcmp(argv[arg], "-i")) || (!strcmp(argv[arg], "--iter-output")))
     {
-        freecell_solver_user_set_iter_handler(
-            instance,
-            my_iter_handler,
-            dc
-            );
+#define set_iter_handler() \
+        freecell_solver_user_set_iter_handler(   \
+            instance,   \
+            my_iter_handler,   \
+            dc    \
+            );        \
         dc->debug_iter_output_on = 1;
+
+        set_iter_handler();
     }
     else if ((!strcmp(argv[arg], "-s")) || (!strcmp(argv[arg], "--state-output")))
     {
+        set_iter_handler();
         dc->debug_iter_state_output = 1;
+#undef set_iter_handler
     }
     else if ((!strcmp(argv[arg], "-p")) || (!strcmp(argv[arg], "--parseable-output")))
     {
@@ -315,6 +499,11 @@ static int cmd_line_callback(
     else if ((!strcmp(argv[arg], "-sn")) || (!strcmp(argv[arg], "--standard-notation")))
     {
         dc->standard_notation = 1;
+
+    }
+    else if ((!strcmp(argv[arg], "-snx")) || (!strcmp(argv[arg], "--standard-notation-extended")))
+    {
+        dc->standard_notation = 2;
     }
     else if ((!strcmp(argv[arg], "-sam")) || (!strcmp(argv[arg], "--display-states-and-moves")))
     {
@@ -325,6 +514,17 @@ static int cmd_line_callback(
     {
         dc->display_parent_iter_num = 1;
     }
+    else if ((!strcmp(argv[arg], "--reset")))
+    {
+        init_debug_context(dc);
+        freecell_solver_user_set_iter_handler(
+            instance,
+            NULL,
+            NULL
+            );
+        *num_to_skip = 0;
+        return FCS_CMD_LINE_OK;
+    }
     else
     {
         printf("Unimplemented option - \"%s\"!", argv[arg]);
@@ -344,7 +544,7 @@ static void select_signal_handler(int signal_num)
 }
 
 static void * current_instance;
-static freecell_solver_display_information_context_t * dc;
+static fc_solve_display_information_context_t * dc;
 
 
 static void command_signal_handler(int signal_num)
@@ -389,6 +589,8 @@ static void command_signal_handler(int signal_num)
 
 static char * known_parameters[] = {
     "-h", "--help",
+        "--help-configs", "--help-options", "--help-problems",
+        "--help-real-help", "--help-short-sol", "--help-summary",
     "-i", "--iter-output",
     "-s", "--state-output",
     "-p", "--parseable-output",
@@ -396,11 +598,16 @@ static char * known_parameters[] = {
     "-t", "--display-10-as-t",
     "-m", "--display-moves",
     "-sn", "--standard-notation",
+    "-snx", "--standard-notation-extended",
     "-sam", "--display-states-and-moves",
     "-pi", "--display-parent-iter",
+    "--reset",
+    "--version",
     NULL
     };
 
+#define USER_STATE_SIZE 1024
+
 int main(int argc, char * argv[])
 {
     int parser_ret;
@@ -408,23 +615,15 @@ int main(int argc, char * argv[])
     char * error_string;
     int arg;
     FILE * file;
-    char user_state[1024];
+    char user_state[USER_STATE_SIZE];
     int ret;
 
-    freecell_solver_display_information_context_t debug_context;
+    fc_solve_display_information_context_t debug_context;
 
-    debug_context.parseable_output = 0;
-    debug_context.canonized_order_output = 0;
-    debug_context.display_10_as_t = 0;
-    debug_context.display_parent_iter_num = 0;
-    debug_context.display_moves = 0;
-    debug_context.display_states =1;
-    debug_context.standard_notation = 0;
+    init_debug_context(&debug_context);
 
     dc = &debug_context;
 
-
-
     instance = freecell_solver_user_alloc();
 
     current_instance = instance;
@@ -449,7 +648,14 @@ int main(int argc, char * argv[])
         return 0;
     }
     else if (
-        (parser_ret == FCS_CMD_LINE_PARAM_WITH_NO_ARG) ||
+        (parser_ret == FCS_CMD_LINE_PARAM_WITH_NO_ARG)
+            )
+    {
+        fprintf(stderr, "The command line parameter \"%s\" requires an argument"
+                " and was not supplied with one.\n", argv[arg]);
+        return (-1);
+    }
+    else if (
         (parser_ret == FCS_CMD_LINE_ERROR_IN_ARG)
         )
     {
@@ -465,6 +671,14 @@ int main(int argc, char * argv[])
     if ((arg == argc) || (!strcmp(argv[arg], "-")))
     {
         file = stdin;
+        if (!getenv("FREECELL_SOLVER_QUIET"))
+        {
+            fprintf(stderr, "%s",
+                    "Reading the board from the standard input.\n"
+                    "Type \"fc-solve --help\" for more usage information.\n"
+                    "To cancel this message set the FREECELL_SOLVER_QUIET environment variable.\n"
+                   );
+        }
     }
     else if (argv[arg][0] == '-')
     {
@@ -492,7 +706,8 @@ int main(int argc, char * argv[])
             return -1;
         }
     }
-    fread(user_state, sizeof(user_state[0]), sizeof(user_state)/sizeof(user_state[0]), file);
+    memset(user_state, '\0', sizeof(user_state));
+    fread(user_state, sizeof(user_state[0]), USER_STATE_SIZE-1, file);
     fclose(file);
 
     /* Win32 Does not have those signals */
@@ -569,7 +784,8 @@ int main(int argc, char * argv[])
                     if (debug_context.display_moves)
                     {
                         as_string =
-                            freecell_solver_user_move_to_string(
+                            freecell_solver_user_move_to_string_w_state(
+                                instance,
                                 move,
                                 debug_context.standard_notation
                                 );
