@@ -32,7 +32,7 @@
    entry in the pqueue depending on whether it is ascending or descending respectively. Finally the bool32 tells you whether
    the list is sorted ascending or descending... */
 
-void freecell_solver_PQueueInitialise(
+void fc_solve_PQueueInitialise(
     PQUEUE *pq,
     int32 MaxElements
     )
@@ -53,7 +53,11 @@ void freecell_solver_PQueueInitialise(
    returns TRUE if succesful, FALSE if fails. (You fail by filling the pqueue.)
    PGetRating is a function which returns the rating of the item you're adding for sorting purposes */
 
-int freecell_solver_PQueuePush( PQUEUE *pq, void *item, pq_rating_t r)
+int fc_solve_PQueuePush(
+        PQUEUE *pq,
+        fcs_state_extra_info_t * val,
+        pq_rating_t r
+        )
 {
     uint32 i;
     pq_element_t * Elements = pq->Elements;
@@ -97,7 +101,7 @@ int freecell_solver_PQueuePush( PQUEUE *pq, void *item, pq_rating_t r)
         }
 
         /* then add the element at the space we created. */
-        Elements[i].item = item;
+        Elements[i].val = val;
         Elements[i].rating = r;
     }
 
@@ -110,14 +114,19 @@ int freecell_solver_PQueuePush( PQUEUE *pq, void *item, pq_rating_t r)
 #define PQueueIsEmpty(pq) ((pq)->CurrentSize == 0)
 
 /* free up memory for pqueue */
-void freecell_solver_PQueueFree( PQUEUE *pq )
+void fc_solve_PQueueFree( PQUEUE *pq )
 {
     free( pq->Elements );
 }
 
-/* remove the first node from the pqueue and provide a pointer to it */
+/* remove the first node from the pqueue and provide a pointer to it
+ *
+ * Returns 0 if found.
+ * Returns 1 if not found.
+ *
+ * */
 
-void *freecell_solver_PQueuePop( PQUEUE *pq)
+int fc_solve_PQueuePop( PQUEUE *pq, fcs_state_extra_info_t * * val)
 {
     int32 i;
     int32 child;
@@ -129,7 +138,8 @@ void *freecell_solver_PQueuePop( PQUEUE *pq)
 
     if( PQueueIsEmpty( pq ) )
     {
-        return NULL;
+        *val = NULL;
+        return 1;
     }
 
     pMaxElement = Elements[PQ_FIRST_ENTRY];
@@ -167,7 +177,8 @@ void *freecell_solver_PQueuePop( PQUEUE *pq)
     Elements[i] = pLastElement;
     pq->CurrentSize = CurrentSize;
 
-    return pMaxElement.item;
+    *val = pMaxElement.val;
+    return 0;
 }
 
 
