@@ -1,13 +1,13 @@
 /*
  * fcs.h - header file of the preset management functions for Freecell Solver.
  *
- * Written by Shlomi Fish (shlomif@vipe.technion.ac.il), 2000
+ * Written by Shlomi Fish ( http://www.shlomifish.org/ ), 2000
  *
  * This file is in the public domain (it's uncopyrighted).
  */
 
-#ifndef __PRESET_H
-#define __PRESET_H
+#ifndef FC_SOLVE__PRESET_H
+#define FC_SOLVE__PRESET_H
 
 #ifdef __cplusplus
 extern "C" {
@@ -32,23 +32,23 @@ struct fcs_preset_struct
 
 typedef struct fcs_preset_struct fcs_preset_t;
 
-extern int freecell_solver_apply_preset_by_ptr(
-    freecell_solver_instance_t * instance,
+extern int fc_solve_apply_preset_by_ptr(
+    fc_solve_instance_t * instance,
     const fcs_preset_t * preset_ptr
     );
 
-extern int freecell_solver_apply_preset_by_name(
-    freecell_solver_instance_t * instance,
+extern int fc_solve_apply_preset_by_name(
+    fc_solve_instance_t * instance,
     const char * name
     );
 
-extern int freecell_solver_apply_tests_order(
+extern int fc_solve_apply_tests_order(
     fcs_tests_order_t * tests_order,
     const char * string,
     char * * error_string
     );
 
-extern int freecell_solver_get_preset_by_name(
+extern int fc_solve_get_preset_by_name(
     const char * name,
     const fcs_preset_t * * preset_ptr
     );
