@@ -1,6 +1,6 @@
 
-#ifndef __RAND_H
-#define __RAND_H
+#ifndef FC_SOLVE__RAND_H
+#define FC_SOLVE__RAND_H
 
 #ifdef __cplusplus
 extern "C" {
@@ -15,10 +15,10 @@ struct fcs_rand_struct
 
 typedef struct fcs_rand_struct fcs_rand_t;
 
-extern fcs_rand_t * freecell_solver_rand_alloc(unsigned int seed);
-extern void freecell_solver_rand_free(fcs_rand_t * rand);
+extern fcs_rand_t * fc_solve_rand_alloc(unsigned int seed);
+extern void fc_solve_rand_free(fcs_rand_t * rand);
 
-extern void freecell_solver_rand_srand(fcs_rand_t * rand, int seed);
+extern void fc_solve_rand_srand(fcs_rand_t * rand, int seed);
 
 static GCC_INLINE int freecell_solver_rand_rand15(fcs_rand_t * rand)
 {
@@ -32,7 +32,7 @@ static GCC_INLINE int freecell_solver_rand_rand15(fcs_rand_t * rand)
  * 15-bit ones.
  *
  * */
-static GCC_INLINE int freecell_solver_rand_get_random_number(fcs_rand_t * rand)
+static GCC_INLINE int fc_solve_rand_get_random_number(fcs_rand_t * rand)
 {
     int one, two;
     one = freecell_solver_rand_rand15(rand);
