@@ -1,7 +1,7 @@
 /*
  * simpsim.c - a module that contains Simple Simon Moves.
  *
- * Written by Shlomi Fish (shlomif@vipe.technion.ac.il), 2001
+ * Written by Shlomi Fish ( http://www.shlomifish.org/ ), 2001
  *
  * This file is in the public domain (it's uncopyrighted).
  */
@@ -12,6 +12,8 @@
 
 #include "tests.h"
 
+#include "ms_ca.h"
+
 #ifdef DMALLOC
 #include "dmalloc.h"
 #endif
@@ -29,32 +31,18 @@
         (fcs_suit_is_ss_true_parent(fcs_card_suit(parent),fcs_card_suit(child))) \
     )
 
-/*
- *  Those are some macros to make it easier for the programmer.
- * */
-#define state_with_locations (*ptr_state_with_locations)
-#define state (ptr_state_with_locations->s)
-#define new_state_with_locations (*ptr_new_state_with_locations)
-#define new_state (ptr_new_state_with_locations->s)
-
-
-
-int freecell_solver_sfs_simple_simon_move_sequence_to_founds(
-        freecell_solver_soft_thread_t * soft_thread,
-        fcs_state_with_locations_t * ptr_state_with_locations,
-        int depth,
+int fc_solve_sfs_simple_simon_move_sequence_to_founds(
+        fc_solve_soft_thread_t * soft_thread,
+        fcs_state_extra_info_t * ptr_state_val,
         int num_freestacks,
         int num_freecells,
         fcs_derived_states_list_t * derived_states_list,
         int reparent
         )
 {
-    freecell_solver_hard_thread_t * hard_thread = soft_thread->hard_thread;
-    freecell_solver_instance_t * instance = hard_thread->instance;
+    tests_declare_accessors();
 
-    fcs_state_with_locations_t * ptr_new_state_with_locations;
 
-    fcs_move_stack_t * moves;
     fcs_move_t temp_move;
 
     int check;
@@ -74,13 +62,17 @@ int freecell_solver_sfs_simple_simon_move_sequence_to_founds(
      * */
     fcs_card_t card, above_card;
 
-    int state_stacks_num;
-    state_stacks_num = instance->stacks_num;
+#ifndef HARD_CODED_NUM_STACKS
+    int stacks_num;
+#endif
+    tests_define_accessors();
+
+#ifndef HARD_CODED_NUM_STACKS
+    stacks_num = instance->stacks_num;
+#endif
 
 
-    fcs_move_stack_alloc_into_var(moves);
-
-    for(stack=0;stack<state_stacks_num;stack++)
+    for(stack=0;stack<LOCAL_STACKS_NUM;stack++)
     {
         cards_num = fcs_stack_len(state, stack);
         if (cards_num >= 13)
@@ -108,6 +100,8 @@ int freecell_solver_sfs_simple_simon_move_sequence_to_founds(
 
                 sfs_check_state_begin();
 
+                my_copy_stack(stack);
+
                 suit = fcs_card_suit(card);
                 for(a=0;a<13;a++)
                 {
@@ -115,7 +109,6 @@ int freecell_solver_sfs_simple_simon_move_sequence_to_founds(
                     fcs_increment_foundation(new_state, suit);
                 }
 
-                fcs_move_stack_reset(moves);
 
                 fcs_move_set_type(temp_move, FCS_MOVE_TYPE_SEQ_TO_FOUNDATION);
                 fcs_move_set_src_stack(temp_move, stack);
@@ -127,27 +120,21 @@ int freecell_solver_sfs_simple_simon_move_sequence_to_founds(
         }
     }
 
-    fcs_move_stack_destroy(moves);
-
     return FCS_STATE_IS_NOT_SOLVEABLE;
 }
 
-int freecell_solver_sfs_simple_simon_move_sequence_to_true_parent(
-        freecell_solver_soft_thread_t * soft_thread,
-        fcs_state_with_locations_t * ptr_state_with_locations,
-        int depth,
+int fc_solve_sfs_simple_simon_move_sequence_to_true_parent(
+        fc_solve_soft_thread_t * soft_thread,
+        fcs_state_extra_info_t * ptr_state_val,
         int num_freestacks,
         int num_freecells,
         fcs_derived_states_list_t * derived_states_list,
         int reparent
         )
 {
-    freecell_solver_hard_thread_t * hard_thread = soft_thread->hard_thread;
-    freecell_solver_instance_t * instance = hard_thread->instance;
+    tests_declare_accessors();
 
-    fcs_state_with_locations_t * ptr_new_state_with_locations;
 
-    fcs_move_stack_t * moves;
     fcs_move_t temp_move;
 
     int check;
@@ -179,13 +166,17 @@ int freecell_solver_sfs_simple_simon_move_sequence_to_true_parent(
      * */
     int card_num, num_true_seqs, ds, dest_cards_num ;
 
-    int state_stacks_num;
-    state_stacks_num = instance->stacks_num;
+#ifndef HARD_CODED_NUM_STACKS
+    int stacks_num;
+#endif
+    tests_define_accessors();
+
+#ifndef HARD_CODED_NUM_STACKS
+    stacks_num = instance->stacks_num;
+#endif
 
 
-    fcs_move_stack_alloc_into_var(moves);
-
-    for(stack=0;stack<state_stacks_num;stack++)
+    for(stack=0;stack<LOCAL_STACKS_NUM;stack++)
     {
         cards_num = fcs_stack_len(state, stack);
         if (cards_num > 0)
@@ -199,7 +190,7 @@ int freecell_solver_sfs_simple_simon_move_sequence_to_true_parent(
 
             for(h=cards_num-2;h>=-1;h--)
             {
-                for(ds=0;ds<state_stacks_num;ds++)
+                for(ds=0;ds<LOCAL_STACKS_NUM;ds++)
                 {
                     if (ds == stack)
                     {
@@ -222,7 +213,9 @@ int freecell_solver_sfs_simple_simon_move_sequence_to_true_parent(
 
                                 sfs_check_state_begin();
 
-                                fcs_move_stack_reset(moves);
+                                my_copy_stack(stack);
+                                my_copy_stack(ds);
+
 
                                 fcs_move_sequence(ds, stack, h+1, cards_num-1, a);
                                 sfs_check_state_end();
@@ -254,27 +247,21 @@ int freecell_solver_sfs_simple_simon_move_sequence_to_true_parent(
         }
     }
 
-    fcs_move_stack_destroy(moves);
-
     return FCS_STATE_IS_NOT_SOLVEABLE;
 }
 
-int freecell_solver_sfs_simple_simon_move_whole_stack_sequence_to_false_parent(
-        freecell_solver_soft_thread_t * soft_thread,
-        fcs_state_with_locations_t * ptr_state_with_locations,
-        int depth,
+int fc_solve_sfs_simple_simon_move_whole_stack_sequence_to_false_parent(
+        fc_solve_soft_thread_t * soft_thread,
+        fcs_state_extra_info_t * ptr_state_val,
         int num_freestacks,
         int num_freecells,
         fcs_derived_states_list_t * derived_states_list,
         int reparent
         )
 {
-    freecell_solver_hard_thread_t * hard_thread = soft_thread->hard_thread;
-    freecell_solver_instance_t * instance = hard_thread->instance;
+    tests_declare_accessors();
 
-    fcs_state_with_locations_t * ptr_new_state_with_locations;
 
-    fcs_move_stack_t * moves;
     fcs_move_t temp_move;
 
     int check;
@@ -296,12 +283,16 @@ int freecell_solver_sfs_simple_simon_move_whole_stack_sequence_to_false_parent(
     fcs_card_t card, temp_card, dest_card;
     int card_num, num_true_seqs, h, ds, dest_cards_num ;
 
-    int state_stacks_num;
-    state_stacks_num = instance->stacks_num;
+#ifndef HARD_CODED_NUM_STACKS
+    int stacks_num;
+#endif
+    tests_define_accessors();
 
-    fcs_move_stack_alloc_into_var(moves);
+#ifndef HARD_CODED_NUM_STACKS
+    stacks_num = instance->stacks_num;
+#endif
 
-    for(stack=0;stack<state_stacks_num;stack++)
+    for(stack=0;stack<LOCAL_STACKS_NUM;stack++)
     {
         cards_num = fcs_stack_len(state, stack);
         if (cards_num > 0)
@@ -334,7 +325,7 @@ int freecell_solver_sfs_simple_simon_move_whole_stack_sequence_to_false_parent(
                 continue;
             }
 
-            for(ds=0;ds<state_stacks_num;ds++)
+            for(ds=0;ds<LOCAL_STACKS_NUM;ds++)
             {
                 dest_cards_num = fcs_stack_len(state,ds);
                 if (dest_cards_num > 0)
@@ -352,7 +343,9 @@ int freecell_solver_sfs_simple_simon_move_whole_stack_sequence_to_false_parent(
 
                             sfs_check_state_begin();
 
-                            fcs_move_stack_reset(moves);
+                            my_copy_stack(stack);
+                            my_copy_stack(ds);
+
 
                             fcs_move_sequence(ds, stack, h+1, cards_num-1, a);
                             sfs_check_state_end();
@@ -365,28 +358,22 @@ int freecell_solver_sfs_simple_simon_move_whole_stack_sequence_to_false_parent(
         }
     }
 
-    fcs_move_stack_destroy(moves);
-
     return FCS_STATE_IS_NOT_SOLVEABLE;
 }
 
 
-int freecell_solver_sfs_simple_simon_move_sequence_to_true_parent_with_some_cards_above(
-        freecell_solver_soft_thread_t * soft_thread,
-        fcs_state_with_locations_t * ptr_state_with_locations,
-        int depth,
+int fc_solve_sfs_simple_simon_move_sequence_to_true_parent_with_some_cards_above(
+        fc_solve_soft_thread_t * soft_thread,
+        fcs_state_extra_info_t * ptr_state_val,
         int num_freestacks,
         int num_freecells,
         fcs_derived_states_list_t * derived_states_list,
         int reparent
         )
 {
-    freecell_solver_hard_thread_t * hard_thread = soft_thread->hard_thread;
-    freecell_solver_instance_t * instance = hard_thread->instance;
+    tests_declare_accessors();
 
-    fcs_state_with_locations_t * ptr_new_state_with_locations;
 
-    fcs_move_stack_t * moves;
     fcs_move_t temp_move;
     int check;
 
@@ -429,12 +416,16 @@ int freecell_solver_sfs_simple_simon_move_sequence_to_true_parent_with_some_card
     int after_junk_num_freestacks;
     int junk_move_to_stacks[MAX_NUM_STACKS];
 
-    int state_stacks_num;
-    state_stacks_num = instance->stacks_num;
+#ifndef HARD_CODED_NUM_STACKS
+    int stacks_num;
+#endif
+    tests_define_accessors();
 
-    fcs_move_stack_alloc_into_var(moves);
+#ifndef HARD_CODED_NUM_STACKS
+    stacks_num = instance->stacks_num;
+#endif
 
-    for(stack=0;stack<state_stacks_num;stack++)
+    for(stack=0;stack<LOCAL_STACKS_NUM;stack++)
     {
         cards_num = fcs_stack_len(state, stack);
         if (cards_num > 0)
@@ -448,7 +439,7 @@ int freecell_solver_sfs_simple_simon_move_sequence_to_true_parent_with_some_card
 
             for(h=cards_num-2;h>=-1;h--)
             {
-                for(ds=0;ds<state_stacks_num;ds++)
+                for(ds=0;ds<LOCAL_STACKS_NUM;ds++)
                 {
                     if (ds == stack)
                     {
@@ -499,7 +490,7 @@ int freecell_solver_sfs_simple_simon_move_sequence_to_true_parent_with_some_card
                                     seq_points[num_separate_false_seqs++] = above_c+1;
                                 }
 
-                                for(a=0;a<state_stacks_num;a++)
+                                for(a=0;a<LOCAL_STACKS_NUM;a++)
                                 {
                                     stacks_map[a] = 0;
                                 }
@@ -520,7 +511,7 @@ int freecell_solver_sfs_simple_simon_move_sequence_to_true_parent_with_some_card
 
                                     /* Let's try to find a suitable parent on top one of the stacks */
                                     for(clear_junk_dest_stack=0;
-                                        clear_junk_dest_stack < state_stacks_num;
+                                        clear_junk_dest_stack < LOCAL_STACKS_NUM;
                                         clear_junk_dest_stack++
                                        )
                                     {
@@ -543,7 +534,7 @@ int freecell_solver_sfs_simple_simon_move_sequence_to_true_parent_with_some_card
                                         }
                                     }
 
-                                    if (clear_junk_dest_stack == state_stacks_num)
+                                    if (clear_junk_dest_stack == LOCAL_STACKS_NUM)
                                     {
                                         clear_junk_dest_stack = -1;
                                     }
@@ -558,7 +549,7 @@ int freecell_solver_sfs_simple_simon_move_sequence_to_true_parent_with_some_card
                                                 /* Find an empty stack and designate it as the destination for the junk */
                                                 for(
                                                     clear_junk_dest_stack = 0;
-                                                    clear_junk_dest_stack < state_stacks_num;
+                                                    clear_junk_dest_stack < LOCAL_STACKS_NUM;
                                                     clear_junk_dest_stack++
                                                    )
                                                 {
@@ -593,7 +584,9 @@ int freecell_solver_sfs_simple_simon_move_sequence_to_true_parent_with_some_card
 
                                         sfs_check_state_begin();
 
-                                        fcs_move_stack_reset(moves);
+                                        my_copy_stack(ds);
+                                        my_copy_stack(stack);
+
 
                                         /* Move the junk cards to their place */
 
@@ -608,6 +601,8 @@ int freecell_solver_sfs_simple_simon_move_sequence_to_true_parent_with_some_card
                                             int start = seq_points[false_seq_index];
                                             int end = ((false_seq_index == 0) ? (dest_cards_num-1) : (seq_points[false_seq_index-1]-1));
 
+                                            my_copy_stack(junk_move_to_stacks[false_seq_index]);
+
                                             fcs_move_sequence(junk_move_to_stacks[false_seq_index], ds, start, end, a);
 
                                         }
@@ -644,27 +639,21 @@ int freecell_solver_sfs_simple_simon_move_sequence_to_true_parent_with_some_card
         }
     }
 
-    fcs_move_stack_destroy(moves);
-
     return FCS_STATE_IS_NOT_SOLVEABLE;
 }
 
-int freecell_solver_sfs_simple_simon_move_sequence_with_some_cards_above_to_true_parent(
-        freecell_solver_soft_thread_t * soft_thread,
-        fcs_state_with_locations_t * ptr_state_with_locations,
-        int depth,
+int fc_solve_sfs_simple_simon_move_sequence_with_some_cards_above_to_true_parent(
+        fc_solve_soft_thread_t * soft_thread,
+        fcs_state_extra_info_t * ptr_state_val,
         int num_freestacks,
         int num_freecells,
         fcs_derived_states_list_t * derived_states_list,
         int reparent
         )
 {
-    freecell_solver_hard_thread_t * hard_thread = soft_thread->hard_thread;
-    freecell_solver_instance_t * instance = hard_thread->instance;
+    tests_declare_accessors();
 
-    fcs_state_with_locations_t * ptr_new_state_with_locations;
 
-    fcs_move_stack_t * moves;
     fcs_move_t temp_move;
 
     int stack, cards_num, suit, a;
@@ -678,12 +667,16 @@ int freecell_solver_sfs_simple_simon_move_sequence_with_some_cards_above_to_true
     int false_seq_index;
     int junk_move_to_stacks[MAX_NUM_CARDS_IN_A_STACK];
 
-    int state_stacks_num;
-    state_stacks_num = instance->stacks_num;
+#ifndef HARD_CODED_NUM_STACKS
+    int stacks_num;
+#endif
+    tests_define_accessors();
 
-    fcs_move_stack_alloc_into_var(moves);
+#ifndef HARD_CODED_NUM_STACKS
+    stacks_num = instance->stacks_num;
+#endif
 
-    for(stack=0;stack<state_stacks_num;stack++)
+    for(stack=0;stack<LOCAL_STACKS_NUM;stack++)
     {
         cards_num = fcs_stack_len(state, stack);
         if (cards_num > 0)
@@ -744,7 +737,7 @@ int freecell_solver_sfs_simple_simon_move_sequence_with_some_cards_above_to_true
                     seq_points[num_separate_false_seqs++] = above_c+1;
                 }
 
-                for(ds=0;ds<state_stacks_num;ds++)
+                for(ds=0;ds<LOCAL_STACKS_NUM;ds++)
                 {
                     if (ds == stack)
                     {
@@ -762,7 +755,7 @@ int freecell_solver_sfs_simple_simon_move_sequence_with_some_cards_above_to_true
                         /* This is a suitable parent - let's check if we
                          * have enough empty stacks to make the move feasible */
 
-                            for(a=0;a<state_stacks_num;a++)
+                            for(a=0;a<LOCAL_STACKS_NUM;a++)
                             {
                                 stacks_map[a] = 0;
                             }
@@ -779,7 +772,7 @@ int freecell_solver_sfs_simple_simon_move_sequence_with_some_cards_above_to_true
 
                                 /* Let's try to find a suitable parent on top one of the stacks */
                                 for(clear_junk_dest_stack=0;
-                                    clear_junk_dest_stack < state_stacks_num;
+                                    clear_junk_dest_stack < LOCAL_STACKS_NUM;
                                     clear_junk_dest_stack++
                                    )
                                 {
@@ -802,7 +795,7 @@ int freecell_solver_sfs_simple_simon_move_sequence_with_some_cards_above_to_true
                                     }
                                 }
 
-                                if (clear_junk_dest_stack == state_stacks_num)
+                                if (clear_junk_dest_stack == LOCAL_STACKS_NUM)
                                 {
                                     clear_junk_dest_stack = -1;
                                 }
@@ -817,7 +810,7 @@ int freecell_solver_sfs_simple_simon_move_sequence_with_some_cards_above_to_true
                                             /* Find an empty stack and designate it as the destination for the junk */
                                             for(
                                                 clear_junk_dest_stack = 0;
-                                                clear_junk_dest_stack < state_stacks_num;
+                                                clear_junk_dest_stack < LOCAL_STACKS_NUM;
                                                 clear_junk_dest_stack++
                                                )
                                             {
@@ -845,7 +838,9 @@ int freecell_solver_sfs_simple_simon_move_sequence_with_some_cards_above_to_true
                                 {
                                     sfs_check_state_begin();
 
-                                    fcs_move_stack_reset(moves);
+                                    my_copy_stack(stack);
+                                    my_copy_stack(ds);
+
 
 
                                     /* Let's boogie - we can move everything */
@@ -868,6 +863,8 @@ int freecell_solver_sfs_simple_simon_move_sequence_with_some_cards_above_to_true
                                             src_stack = stack;
                                         }
 
+                                        my_copy_stack(junk_move_to_stacks[false_seq_index]);
+
                                         fcs_move_sequence(junk_move_to_stacks[false_seq_index], src_stack, start, end, a);
                                     }
 
@@ -883,27 +880,21 @@ int freecell_solver_sfs_simple_simon_move_sequence_with_some_cards_above_to_true
         }
     }
 
-    fcs_move_stack_destroy(moves);
-
     return FCS_STATE_IS_NOT_SOLVEABLE;
 }
 
-int freecell_solver_sfs_simple_simon_move_sequence_with_junk_seq_above_to_true_parent_with_some_cards_above(
-        freecell_solver_soft_thread_t * soft_thread,
-        fcs_state_with_locations_t * ptr_state_with_locations,
-        int depth,
+int fc_solve_sfs_simple_simon_move_sequence_with_junk_seq_above_to_true_parent_with_some_cards_above(
+        fc_solve_soft_thread_t * soft_thread,
+        fcs_state_extra_info_t * ptr_state_val,
         int num_freestacks,
         int num_freecells,
         fcs_derived_states_list_t * derived_states_list,
         int reparent
         )
 {
-    freecell_solver_hard_thread_t * hard_thread = soft_thread->hard_thread;
-    freecell_solver_instance_t * instance = hard_thread->instance;
+    tests_declare_accessors();
 
-    fcs_state_with_locations_t * ptr_new_state_with_locations;
 
-    fcs_move_stack_t * moves;
     fcs_move_t temp_move;
 
     int check;
@@ -948,12 +939,16 @@ int freecell_solver_sfs_simple_simon_move_sequence_with_junk_seq_above_to_true_p
     int end_of_junk;
     int num_true_seqs;
 
-    int state_stacks_num;
-    state_stacks_num = instance->stacks_num;
+#ifndef HARD_CODED_NUM_STACKS
+    int stacks_num;
+#endif
+    tests_define_accessors();
 
-    fcs_move_stack_alloc_into_var(moves);
+#ifndef HARD_CODED_NUM_STACKS
+    stacks_num = instance->stacks_num;
+#endif
 
-    for(stack=0;stack<state_stacks_num;stack++)
+    for(stack=0;stack<LOCAL_STACKS_NUM;stack++)
     {
         cards_num = fcs_stack_len(state, stack);
         if (cards_num > 0)
@@ -1010,7 +1005,7 @@ int freecell_solver_sfs_simple_simon_move_sequence_with_junk_seq_above_to_true_p
                 card_num = fcs_card_card_num(card);
                 suit = fcs_card_suit(card);
 
-                for(ds=0;ds<state_stacks_num;ds++)
+                for(ds=0;ds<LOCAL_STACKS_NUM;ds++)
                 {
                     if (ds == stack)
                     {
@@ -1058,7 +1053,7 @@ int freecell_solver_sfs_simple_simon_move_sequence_with_junk_seq_above_to_true_p
                                     seq_points[num_separate_false_seqs++] = above_c+1;
                                 }
 
-                                for(a=0;a<state_stacks_num;a++)
+                                for(a=0;a<LOCAL_STACKS_NUM;a++)
                                 {
                                     stacks_map[a] = 0;
                                 }
@@ -1082,13 +1077,13 @@ int freecell_solver_sfs_simple_simon_move_sequence_with_junk_seq_above_to_true_p
                                     int the_num_true_seqs =
                                         (
                                             (false_seq_index == num_separate_false_seqs) ?
-                                                1 :
+                                                num_src_junk_true_seqs :
                                                 above_num_true_seqs[false_seq_index]
                                         );
 
                                     /* Let's try to find a suitable parent on top one of the stacks */
                                     for(clear_junk_dest_stack=0;
-                                        clear_junk_dest_stack < state_stacks_num;
+                                        clear_junk_dest_stack < LOCAL_STACKS_NUM;
                                         clear_junk_dest_stack++
                                        )
                                     {
@@ -1111,7 +1106,7 @@ int freecell_solver_sfs_simple_simon_move_sequence_with_junk_seq_above_to_true_p
                                         }
                                     }
 
-                                    if (clear_junk_dest_stack == state_stacks_num)
+                                    if (clear_junk_dest_stack == LOCAL_STACKS_NUM)
                                     {
                                         clear_junk_dest_stack = -1;
                                     }
@@ -1126,7 +1121,7 @@ int freecell_solver_sfs_simple_simon_move_sequence_with_junk_seq_above_to_true_p
                                                 /* Find an empty stack and designate it as the destination for the junk */
                                                 for(
                                                     clear_junk_dest_stack = 0;
-                                                    clear_junk_dest_stack < state_stacks_num;
+                                                    clear_junk_dest_stack < LOCAL_STACKS_NUM;
                                                     clear_junk_dest_stack++
                                                    )
                                                 {
@@ -1156,7 +1151,9 @@ int freecell_solver_sfs_simple_simon_move_sequence_with_junk_seq_above_to_true_p
 
                                         sfs_check_state_begin();
 
-                                        fcs_move_stack_reset(moves);
+                                        my_copy_stack(stack);
+                                        my_copy_stack(ds);
+
 
                                         /* Move the junk cards to their place */
 
@@ -1183,6 +1180,10 @@ int freecell_solver_sfs_simple_simon_move_sequence_with_junk_seq_above_to_true_p
                                                 src_stack = ds;
                                             }
 
+                                            my_copy_stack(src_stack);
+
+                                            my_copy_stack(junk_move_to_stacks[false_seq_index]);
+
                                             fcs_move_sequence(junk_move_to_stacks[false_seq_index], src_stack, start, end, a);
                                         }
 
@@ -1200,27 +1201,21 @@ int freecell_solver_sfs_simple_simon_move_sequence_with_junk_seq_above_to_true_p
         }
     }
 
-    fcs_move_stack_destroy(moves);
-
     return FCS_STATE_IS_NOT_SOLVEABLE;
 }
 
-int freecell_solver_sfs_simple_simon_move_whole_stack_sequence_to_false_parent_with_some_cards_above(
-        freecell_solver_soft_thread_t * soft_thread,
-        fcs_state_with_locations_t * ptr_state_with_locations,
-        int depth,
+int fc_solve_sfs_simple_simon_move_whole_stack_sequence_to_false_parent_with_some_cards_above(
+        fc_solve_soft_thread_t * soft_thread,
+        fcs_state_extra_info_t * ptr_state_val,
         int num_freestacks,
         int num_freecells,
         fcs_derived_states_list_t * derived_states_list,
         int reparent
         )
 {
-    freecell_solver_hard_thread_t * hard_thread = soft_thread->hard_thread;
-    freecell_solver_instance_t * instance = hard_thread->instance;
+    tests_declare_accessors();
 
-    fcs_state_with_locations_t * ptr_new_state_with_locations;
 
-    fcs_move_stack_t * moves;
     fcs_move_t temp_move;
 
     int check;
@@ -1262,12 +1257,16 @@ int freecell_solver_sfs_simple_simon_move_whole_stack_sequence_to_false_parent_w
     int false_seq_index;
     int junk_move_to_stacks[MAX_NUM_STACKS];
 
-    int state_stacks_num;
-    state_stacks_num = instance->stacks_num;
+#ifndef HARD_CODED_NUM_STACKS
+    int stacks_num;
+#endif
+    tests_define_accessors();
 
-    fcs_move_stack_alloc_into_var(moves);
+#ifndef HARD_CODED_NUM_STACKS
+    stacks_num = instance->stacks_num;
+#endif
 
-    for(stack=0;stack<state_stacks_num;stack++)
+    for(stack=0;stack<LOCAL_STACKS_NUM;stack++)
     {
         cards_num = fcs_stack_len(state, stack);
         if (cards_num > 0)
@@ -1297,7 +1296,7 @@ int freecell_solver_sfs_simple_simon_move_whole_stack_sequence_to_false_parent_w
             }
             if (h == -1)
             {
-                for(ds=0;ds<state_stacks_num;ds++)
+                for(ds=0;ds<LOCAL_STACKS_NUM;ds++)
                 {
                     dest_cards_num = fcs_stack_len(state,ds);
                     if (dest_cards_num > 0)
@@ -1336,7 +1335,7 @@ int freecell_solver_sfs_simple_simon_move_whole_stack_sequence_to_false_parent_w
                                     seq_points[num_separate_false_seqs++] = above_c+1;
                                 }
 
-                                for(a=0;a<state_stacks_num;a++)
+                                for(a=0;a<LOCAL_STACKS_NUM;a++)
                                 {
                                     stacks_map[a] = 0;
                                 }
@@ -1361,7 +1360,7 @@ int freecell_solver_sfs_simple_simon_move_whole_stack_sequence_to_false_parent_w
 
                                     /* Let's try to find a suitable parent on top one of the stacks */
                                     for(clear_junk_dest_stack=0;
-                                        clear_junk_dest_stack < state_stacks_num;
+                                        clear_junk_dest_stack < LOCAL_STACKS_NUM;
                                         clear_junk_dest_stack++
                                        )
                                     {
@@ -1384,7 +1383,7 @@ int freecell_solver_sfs_simple_simon_move_whole_stack_sequence_to_false_parent_w
                                         }
                                     }
 
-                                    if (clear_junk_dest_stack == state_stacks_num)
+                                    if (clear_junk_dest_stack == LOCAL_STACKS_NUM)
                                     {
                                         clear_junk_dest_stack = -1;
                                     }
@@ -1406,7 +1405,9 @@ int freecell_solver_sfs_simple_simon_move_whole_stack_sequence_to_false_parent_w
 
                                         sfs_check_state_begin();
 
-                                        fcs_move_stack_reset(moves);
+                                        my_copy_stack(stack);
+                                        my_copy_stack(ds);
+
 
                                         /* Move the junk cards to their place */
 
@@ -1426,6 +1427,9 @@ int freecell_solver_sfs_simple_simon_move_whole_stack_sequence_to_false_parent_w
                                                 src_stack = ds;
                                             }
 
+                                            my_copy_stack(src_stack);
+                                            my_copy_stack(junk_move_to_stacks[false_seq_index]);
+
                                             fcs_move_sequence( junk_move_to_stacks[false_seq_index], src_stack, start, end, a);
                                         }
 
@@ -1443,27 +1447,21 @@ int freecell_solver_sfs_simple_simon_move_whole_stack_sequence_to_false_parent_w
         }
     }
 
-    fcs_move_stack_destroy(moves);
-
     return FCS_STATE_IS_NOT_SOLVEABLE;
 }
 
-int freecell_solver_sfs_simple_simon_move_sequence_to_parent_on_the_same_stack(
-        freecell_solver_soft_thread_t * soft_thread,
-        fcs_state_with_locations_t * ptr_state_with_locations,
-        int depth,
+int fc_solve_sfs_simple_simon_move_sequence_to_parent_on_the_same_stack(
+        fc_solve_soft_thread_t * soft_thread,
+        fcs_state_extra_info_t * ptr_state_val,
         int num_freestacks,
         int num_freecells,
         fcs_derived_states_list_t * derived_states_list,
         int reparent
         )
 {
-    freecell_solver_hard_thread_t * hard_thread = soft_thread->hard_thread;
-    freecell_solver_instance_t * instance = hard_thread->instance;
+    tests_declare_accessors();
 
-    fcs_state_with_locations_t * ptr_new_state_with_locations;
 
-    fcs_move_stack_t * moves;
     fcs_move_t temp_move;
 
     int check;
@@ -1477,12 +1475,16 @@ int freecell_solver_sfs_simple_simon_move_sequence_to_parent_on_the_same_stack(
 
     fcs_card_t temp_card;
 
-    int state_stacks_num;
-    state_stacks_num = instance->stacks_num;
+#ifndef HARD_CODED_NUM_STACKS
+    int stacks_num;
+#endif
+    tests_define_accessors();
 
-    fcs_move_stack_alloc_into_var(moves);
+#ifndef HARD_CODED_NUM_STACKS
+    stacks_num = instance->stacks_num;
+#endif
 
-    for(stack=0 ; stack < state_stacks_num ; stack++)
+    for(stack=0 ; stack < LOCAL_STACKS_NUM ; stack++)
     {
         cards_num = fcs_stack_len(state, stack);
         if (cards_num > 2)
@@ -1604,7 +1606,7 @@ int freecell_solver_sfs_simple_simon_move_sequence_to_parent_on_the_same_stack(
 
 
 
-                        for(a = 0 ; a < state_stacks_num ; a++)
+                        for(a = 0 ; a < LOCAL_STACKS_NUM ; a++)
                         {
                             stacks_map[a] = 0;
                         }
@@ -1620,7 +1622,7 @@ int freecell_solver_sfs_simple_simon_move_sequence_to_parent_on_the_same_stack(
 
                             /* Let's try to find a suitable parent on top one of the stacks */
                             for(clear_junk_dest_stack=0;
-                                clear_junk_dest_stack < state_stacks_num;
+                                clear_junk_dest_stack < LOCAL_STACKS_NUM;
                                 clear_junk_dest_stack++
                                )
                             {
@@ -1643,7 +1645,7 @@ int freecell_solver_sfs_simple_simon_move_sequence_to_parent_on_the_same_stack(
                                 }
                             }
 
-                            if (clear_junk_dest_stack == state_stacks_num)
+                            if (clear_junk_dest_stack == LOCAL_STACKS_NUM)
                             {
                                 clear_junk_dest_stack = -1;
                             }
@@ -1658,7 +1660,7 @@ int freecell_solver_sfs_simple_simon_move_sequence_to_parent_on_the_same_stack(
                                         /* Find an empty stack and designate it as the destination for the junk */
                                         for(
                                             clear_junk_dest_stack = 0;
-                                            clear_junk_dest_stack < state_stacks_num;
+                                            clear_junk_dest_stack < LOCAL_STACKS_NUM;
                                             clear_junk_dest_stack++
                                            )
                                         {
@@ -1689,10 +1691,10 @@ int freecell_solver_sfs_simple_simon_move_sequence_to_parent_on_the_same_stack(
 
                                 sfs_check_state_begin();
 
-                                fcs_move_stack_reset(moves);
-
                                 /* Move the junk cards to their place */
 
+                                my_copy_stack(stack);
+
                                 for(false_seq_index=0;
                                     false_seq_index<num_separate_false_seqs;
                                     false_seq_index++
@@ -1701,6 +1703,8 @@ int freecell_solver_sfs_simple_simon_move_sequence_to_parent_on_the_same_stack(
                                     int start = seq_points[false_seq_index];
                                     int end = ((false_seq_index == 0) ? (cards_num-1) : (seq_points[false_seq_index-1]-1));
 
+                                    my_copy_stack(junk_move_to_stacks[false_seq_index]);
+
                                     fcs_move_sequence ( junk_move_to_stacks[false_seq_index], stack, start, end, a);
                                 }
 
@@ -1708,6 +1712,8 @@ int freecell_solver_sfs_simple_simon_move_sequence_to_parent_on_the_same_stack(
                                     int end = fcs_stack_len(new_state, junk_move_to_stacks[child_seq_index])-1;
                                     int start = end-(end_of_child_seq-cc);
 
+                                    my_copy_stack(junk_move_to_stacks[child_seq_index]);
+
 
                                     fcs_move_sequence( stack, junk_move_to_stacks[child_seq_index], start, end, a);
                                 }
@@ -1721,8 +1727,6 @@ int freecell_solver_sfs_simple_simon_move_sequence_to_parent_on_the_same_stack(
         }
     }
 
-    fcs_move_stack_destroy(moves);
-
     return FCS_STATE_IS_NOT_SOLVEABLE;
 }
 
