@@ -1,7 +1,7 @@
 /*
  * state.c - state functions module for Freecell Solver
  *
- * Written by Shlomi Fish (shlomif@vipe.technion.ac.il), 2000
+ * Written by Shlomi Fish ( http://www.shlomifish.org/ ), 2000
  *
  * This file is in the public domain (it's uncopyrighted).
  */
@@ -28,11 +28,11 @@
 
 #ifdef DEBUG_STATES
 
-fcs_card_t freecell_solver_empty_card = {0,0};
+fcs_card_t fc_solve_empty_card = {0,0};
 
 #elif defined(COMPACT_STATES) || defined (INDIRECT_STACK_STATES)
 
-fcs_card_t freecell_solver_empty_card = (fcs_card_t)0;
+fcs_card_t fc_solve_empty_card = (fcs_card_t)0;
 
 #endif
 
@@ -84,18 +84,7 @@ static int fcs_stack_compare(const void * s1, const void * s2)
 }
 #elif defined(INDIRECT_STACK_STATES)
 
-
-#if MAX_NUM_DECKS == 1
-static int fcs_stack_compare_for_stack_sort(const void * s1, const void * s2)
-{
-    fcs_card_t card1 = ((fcs_card_t*)s1)[1];
-    fcs_card_t card2 = ((fcs_card_t*)s2)[1];
-
-    return fcs_card_compare(&card1, &card2);
-}
-#endif
-
-int freecell_solver_stack_compare_for_comparison(const void * v_s1, const void * v_s2)
+int fc_solve_stack_compare_for_comparison(const void * v_s1, const void * v_s2)
 {
     const fcs_card_t * s1 = (const fcs_card_t *)v_s1;
     const fcs_card_t * s2 = (const fcs_card_t *)v_s2;
@@ -157,7 +146,7 @@ static int fcs_talon_compare_with_context(const void * p1, const void * p2, fcs_
 #endif
 
 #ifdef DEBUG_STATES
-void freecell_solver_canonize_state(fcs_state_with_locations_t * state, int freecells_num, int stacks_num)
+void fc_solve_canonize_state(fcs_state_with_locations_t * state, int freecells_num, int stacks_num)
 {
     int b,c;
 
@@ -217,8 +206,8 @@ void freecell_solver_canonize_state(fcs_state_with_locations_t * state, int free
 
 #elif defined(COMPACT_STATES)
 
-void freecell_solver_canonize_state(
-    fcs_state_with_locations_t * state,
+void fc_solve_canonize_state(
+    fcs_state_extra_info_t * state_val,
     int freecells_num,
     int stacks_num)
 {
@@ -228,6 +217,8 @@ void freecell_solver_canonize_state(
     fcs_card_t temp_freecell;
     char temp_loc;
 
+    fcs_state_t * state_key = state_val->key;
+
     /* Insertion-sort the stacks */
 
     for(b=1;b<stacks_num;b++)
@@ -236,18 +227,18 @@ void freecell_solver_canonize_state(
         while(
             (c>0)    &&
             (fcs_stack_compare(
-                state->s.data+c*(MAX_NUM_CARDS_IN_A_STACK+1),
-                state->s.data+(c-1)*(MAX_NUM_CARDS_IN_A_STACK+1)
+                state_key->data+c*(MAX_NUM_CARDS_IN_A_STACK+1),
+                state_key->data+(c-1)*(MAX_NUM_CARDS_IN_A_STACK+1)
                 ) < 0)
             )
         {
-            memcpy(temp_stack, state->s.data+c*(MAX_NUM_CARDS_IN_A_STACK+1), (MAX_NUM_CARDS_IN_A_STACK+1));
-            memcpy(state->s.data+c*(MAX_NUM_CARDS_IN_A_STACK+1), state->s.data+(c-1)*(MAX_NUM_CARDS_IN_A_STACK+1), (MAX_NUM_CARDS_IN_A_STACK+1));
-            memcpy(state->s.data+(c-1)*(MAX_NUM_CARDS_IN_A_STACK+1), temp_stack, (MAX_NUM_CARDS_IN_A_STACK+1));
+            memcpy(temp_stack, state_key->data+c*(MAX_NUM_CARDS_IN_A_STACK+1), (MAX_NUM_CARDS_IN_A_STACK+1));
+            memcpy(state_key->data+c*(MAX_NUM_CARDS_IN_A_STACK+1), state_key->data+(c-1)*(MAX_NUM_CARDS_IN_A_STACK+1), (MAX_NUM_CARDS_IN_A_STACK+1));
+            memcpy(state_key->data+(c-1)*(MAX_NUM_CARDS_IN_A_STACK+1), temp_stack, (MAX_NUM_CARDS_IN_A_STACK+1));
 
-            temp_loc = state->stack_locs[c];
-            state->stack_locs[c] = state->stack_locs[c-1];
-            state->stack_locs[c-1] = temp_loc;
+            temp_loc = state_val->stack_locs[c];
+            state_val->stack_locs[c] = state_val->stack_locs[c-1];
+            state_val->stack_locs[c-1] = temp_loc;
 
             c--;
         }
@@ -262,26 +253,26 @@ void freecell_solver_canonize_state(
         while(
             (c>0)    &&
             (fcs_card_compare(
-                state->s.data+FCS_FREECELLS_OFFSET+c,
-                state->s.data+FCS_FREECELLS_OFFSET+c-1
+                state_key->data+FCS_FREECELLS_OFFSET+c,
+                state_key->data+FCS_FREECELLS_OFFSET+c-1
                 ) < 0)
             )
         {
-            temp_freecell = (state->s.data[FCS_FREECELLS_OFFSET+c]);
-            state->s.data[FCS_FREECELLS_OFFSET+c] = state->s.data[FCS_FREECELLS_OFFSET+c-1];
-            state->s.data[FCS_FREECELLS_OFFSET+c-1] = temp_freecell;
+            temp_freecell = (state_key->data[FCS_FREECELLS_OFFSET+c]);
+            state_key->data[FCS_FREECELLS_OFFSET+c] = state_key->data[FCS_FREECELLS_OFFSET+c-1];
+            state_key->data[FCS_FREECELLS_OFFSET+c-1] = temp_freecell;
 
-            temp_loc = state->fc_locs[c];
-            state->fc_locs[c] = state->fc_locs[c-1];
-            state->fc_locs[c-1] = temp_loc;
+            temp_loc = state_val->fc_locs[c];
+            state_val->fc_locs[c] = state_val->fc_locs[c-1];
+            state_val->fc_locs[c-1] = temp_loc;
 
             c--;
         }
     }
 }
 #elif defined(INDIRECT_STACK_STATES)
-void freecell_solver_canonize_state(
-    fcs_state_with_locations_t * state,
+void fc_solve_canonize_state(
+    fcs_state_extra_info_t * state_val,
     int freecells_num,
     int stacks_num)
 {
@@ -290,6 +281,8 @@ void freecell_solver_canonize_state(
     fcs_card_t temp_freecell;
     char temp_loc;
 
+    fcs_state_t * state_key = state_val->key;
+
     /* Insertion-sort the stacks */
     for(b=1;b<stacks_num;b++)
     {
@@ -297,26 +290,22 @@ void freecell_solver_canonize_state(
         while(
             (c>0) &&
             (
-#if MAX_NUM_DECKS > 1
-                freecell_solver_stack_compare_for_comparison
-#else
-                fcs_stack_compare_for_stack_sort
-#endif
+                fc_solve_stack_compare_for_comparison
                 (
-                    (state->s.stacks[c]),
-                    (state->s.stacks[c-1])
+                    (state_key->stacks[c]),
+                    (state_key->stacks[c-1])
                 )
                 < 0
             )
         )
         {
-            temp_stack = state->s.stacks[c];
-            state->s.stacks[c] = state->s.stacks[c-1];
-            state->s.stacks[c-1] = temp_stack;
+            temp_stack = state_key->stacks[c];
+            state_key->stacks[c] = state_key->stacks[c-1];
+            state_key->stacks[c-1] = temp_stack;
 
-            temp_loc = state->stack_locs[c];
-            state->stack_locs[c] = state->stack_locs[c-1];
-            state->stack_locs[c-1] = temp_loc;
+            temp_loc = state_val->stack_locs[c];
+            state_val->stack_locs[c] = state_val->stack_locs[c-1];
+            state_val->stack_locs[c-1] = temp_loc;
 
             c--;
         }
@@ -330,18 +319,18 @@ void freecell_solver_canonize_state(
         while(
             (c>0)     &&
             (fcs_card_compare(
-                &(state->s.freecells[c]),
-                &(state->s.freecells[c-1])
+                &(state_key->freecells[c]),
+                &(state_key->freecells[c-1])
                 ) < 0)
             )
         {
-            temp_freecell = state->s.freecells[c];
-            state->s.freecells[c] = state->s.freecells[c-1];
-            state->s.freecells[c-1] = temp_freecell;
+            temp_freecell = state_key->freecells[c];
+            state_key->freecells[c] = state_key->freecells[c-1];
+            state_key->freecells[c-1] = temp_freecell;
 
-            temp_loc = state->fc_locs[c];
-            state->fc_locs[c] = state->fc_locs[c-1];
-            state->fc_locs[c-1] = temp_loc;
+            temp_loc = state_val->fc_locs[c];
+            state_val->fc_locs[c] = state_val->fc_locs[c-1];
+            state_val->fc_locs[c-1] = temp_loc;
 
             c--;
         }
@@ -351,7 +340,8 @@ void freecell_solver_canonize_state(
 #endif
 
 static void fcs_state_init(
-    fcs_state_with_locations_t * state, 
+    fcs_state_t * state_key,
+    fcs_state_extra_info_t * state_val,
     int stacks_num
 #ifdef INDIRECT_STACK_STATES
     ,char * indirect_stacks_buffer
@@ -359,42 +349,53 @@ static void fcs_state_init(
     )
 {
     int a;
-    memset((void*)&(state->s), 0, sizeof(fcs_state_t));
+    memset(state_key, 0, sizeof(*state_key));
     for(a=0;a<MAX_NUM_STACKS;a++)
     {
-        state->stack_locs[a] = a;
+        state_val->stack_locs[a] = a;
     }
 #ifdef INDIRECT_STACK_STATES
     for(a=0;a<stacks_num;a++)
     {
-        state->s.stacks[a] = &indirect_stacks_buffer[a << 7];
-        memset(state->s.stacks[a], '\0', MAX_NUM_DECKS*52+1);
+        state_key->stacks[a] = &indirect_stacks_buffer[a << 7];
+        memset(state_key->stacks[a], '\0', MAX_NUM_DECKS*52+1);
     }
     for(;a<MAX_NUM_STACKS;a++)
     {
-        state->s.stacks[a] = NULL;
+        state_key->stacks[a] = NULL;
     }
 #endif
     for(a=0;a<MAX_NUM_FREECELLS;a++)
     {
-        state->fc_locs[a] = a;
+        state_val->fc_locs[a] = a;
     }
+    state_val->key = state_key;
+    state_val->parent_val = NULL;
+    state_val->moves_to_parent = NULL;
+    state_val->depth = 0;
+    state_val->visited = 0;
+    state_val->visited_iter = 0;
+    state_val->num_active_children = 0;
+    memset(state_val->scan_visited, '\0', sizeof(state_val->scan_visited));
+#ifdef INDIRECT_STACK_STATES
+    state_val->stacks_copy_on_write_flags = 0;
+#endif
 }
 
 
 #if (FCS_STATE_STORAGE != FCS_STATE_STORAGE_INDIRECT)
-int freecell_solver_state_compare(const void * s1, const void * s2)
+int fc_solve_state_compare(const void * s1, const void * s2)
 {
     return memcmp(s1,s2,sizeof(fcs_state_t));
 }
 
-int freecell_solver_state_compare_equal(const void * s1, const void * s2)
+int fc_solve_state_compare_equal(const void * s1, const void * s2)
 {
     return (!memcmp(s1,s2,sizeof(fcs_state_t)));
 }
 
 
-int freecell_solver_state_compare_with_context(
+int fc_solve_state_compare_with_context(
     const void * s1,
     const void * s2,
     fcs_compare_context_t context
@@ -402,13 +403,27 @@ int freecell_solver_state_compare_with_context(
 {
     return memcmp(s1,s2,sizeof(fcs_state_t));
 }
+
+int fc_solve_state_extra_info_compare_with_context(
+    const void * s1,
+    const void * s2,
+    fcs_compare_context_t context
+    )
+{
+    return
+        memcmp(
+            (((fcs_state_extra_info_t *)s1)->key),
+            (((fcs_state_extra_info_t *)s2)->key),
+            sizeof(fcs_state_t)
+        );
+}
 #else
-int freecell_solver_state_compare_indirect(const void * s1, const void * s2)
+int fc_solve_state_compare_indirect(const void * s1, const void * s2)
 {
     return memcmp(*(fcs_state_with_locations_t * *)s1, *(fcs_state_with_locations_t * *)s2, sizeof(fcs_state_t));
 }
 
-int freecell_solver_state_compare_indirect_with_context(const void * s1, const void * s2, void * context)
+int fc_solve_state_compare_indirect_with_context(const void * s1, const void * s2, void * context)
 {
     return memcmp(*(fcs_state_with_locations_t * *)s1, *(fcs_state_with_locations_t * *)s2, sizeof(fcs_state_t));
 }
@@ -423,8 +438,10 @@ static const char * const num_redeals_prefixes[] = { "Num-Redeals:", "Readels-Nu
 #define strncasecmp(a,b,c) (strnicmp((a),(b),(c)))
 #endif
 
-fcs_state_with_locations_t freecell_solver_initial_user_state_to_c(
+int fc_solve_initial_user_state_to_c(
     const char * string,
+    fcs_state_t * out_state_key,
+    fcs_state_extra_info_t * out_state_val,
     int freecells_num,
     int stacks_num,
     int decks_num
@@ -436,8 +453,6 @@ fcs_state_with_locations_t freecell_solver_initial_user_state_to_c(
 #endif
     )
 {
-    fcs_state_with_locations_t ret_with_locations;
-
     int s,c;
     const char * str;
     fcs_card_t card;
@@ -449,17 +464,26 @@ fcs_state_with_locations_t freecell_solver_initial_user_state_to_c(
     int decks_index[4];
 
     fcs_state_init(
-        &ret_with_locations, 
+        out_state_key,
+        out_state_val,
         stacks_num
-#ifdef INDIRECT_STACK_STATES        
-        , indirect_stacks_buffer  
+#ifdef INDIRECT_STACK_STATES
+        , indirect_stacks_buffer
 #endif
         );
     str = string;
 
     first_line = 1;
 
-#define ret (ret_with_locations.s)
+#define ret (*out_state_key)
+/* Handle the end of string - shouldn't happen */
+#define handle_eos() \
+    { \
+        if ((*str) == '\0') \
+        {  \
+            return FCS_USER_STATE_TO_C__PREMATURE_END_OF_INPUT; \
+        } \
+    }
 
 #ifdef FCS_WITH_TALONS
     if (talon_type == FCS_TALON_KLONDIKE)
@@ -475,6 +499,7 @@ fcs_state_with_locations_t freecell_solver_initial_user_state_to_c(
         {
             while((*str) != '\n')
             {
+                handle_eos();
                 str++;
             }
             str++;
@@ -510,6 +535,7 @@ fcs_state_with_locations_t freecell_solver_initial_user_state_to_c(
                             ((*str) != '\r')
                          )
                     {
+                        handle_eos();
                         str++;
                     }
                     if ((*str == '\n') || (*str == '\r'))
@@ -538,8 +564,9 @@ fcs_state_with_locations_t freecell_solver_initial_user_state_to_c(
                 fcs_put_card_in_freecell(ret, c, card);
             }
 
-            while ((*str != '\n') && (*str != '\0'))
+            while (*str != '\n')
             {
+                handle_eos();
                 str++;
             }
             s--;
@@ -589,6 +616,7 @@ fcs_state_with_locations_t freecell_solver_initial_user_state_to_c(
                         (*str != '\r')
                       )
                 {
+                    handle_eos();
                     str++;
                 }
 
@@ -637,6 +665,7 @@ fcs_state_with_locations_t freecell_solver_initial_user_state_to_c(
                         ((*str) != '\r')
                     )
                     {
+                        handle_eos();
                         str++;
                     }
                     if ((*str == '\n') || (*str == '\r'))
@@ -691,6 +720,7 @@ fcs_state_with_locations_t freecell_solver_initial_user_state_to_c(
         {
             while ((*str < '0') && (*str > '9') && (*str != '\n'))
             {
+                handle_eos();
                 str++;
             }
             if (*str != '\n')
@@ -724,6 +754,7 @@ fcs_state_with_locations_t freecell_solver_initial_user_state_to_c(
                     ((*str) != '\r')
                 )
                 {
+                    handle_eos();
                     str++;
                 }
                 if ((*str == '\n') || (*str == '\r'))
@@ -746,13 +777,14 @@ fcs_state_with_locations_t freecell_solver_initial_user_state_to_c(
         }
     }
 
-    return ret_with_locations;
+    return FCS_USER_STATE_TO_C__SUCCESS;
 }
 
 #undef ret
+#undef handle_eos
 
-int freecell_solver_check_state_validity(
-    fcs_state_with_locations_t * state_with_locations,
+int fc_solve_check_state_validity(
+    fcs_state_extra_info_t * state_val,
     int freecells_num,
     int stacks_num,
     int decks_num,
@@ -766,7 +798,7 @@ int freecell_solver_check_state_validity(
 
     fcs_state_t * state;
 
-    state = (&(state_with_locations->s));
+    state = state_val->key;
 
     /* Initialize all cards to 0 */
     for(d=0;d<4;d++)
@@ -854,8 +886,8 @@ int freecell_solver_check_state_validity(
 #undef state
 
 
-char * freecell_solver_state_as_string(
-    fcs_state_with_locations_t * state_with_locations,
+char * fc_solve_state_as_string(
+    fcs_state_extra_info_t * state_val,
     int freecells_num,
     int stacks_num,
     int decks_num,
@@ -871,12 +903,12 @@ char * freecell_solver_state_as_string(
 
     char str2[128], str3[128], * str2_ptr, * str3_ptr;
 
-    freecell_solver_append_string_t * app_str;
+    fc_solve_append_string_t * app_str;
 
     int stack_locs[MAX_NUM_STACKS];
     int freecell_locs[MAX_NUM_FREECELLS];
 
-    state = (&(state_with_locations->s));
+    state = state_val->key;
 
     if (canonized_order_output)
     {
@@ -893,11 +925,11 @@ char * freecell_solver_state_as_string(
     {
         for(a=0;a<stacks_num;a++)
         {
-            stack_locs[(int)(state_with_locations->stack_locs[a])] = a;
+            stack_locs[(int)(state_val->stack_locs[a])] = a;
         }
         for(a=0;a<freecells_num;a++)
         {
-            freecell_locs[(int)(state_with_locations->fc_locs[a])] = a;
+            freecell_locs[(int)(state_val->fc_locs[a])] = a;
         }
     }
 
@@ -914,7 +946,7 @@ char * freecell_solver_state_as_string(
             decks[a][0] = '0';
     }
 
-    app_str = freecell_solver_append_string_alloc(512);
+    app_str = fc_solve_append_string_alloc(512);
 
     if(!parseable_output)
     {
@@ -938,7 +970,7 @@ char * freecell_solver_state_as_string(
             }
             if (a < decks_num)
             {
-                freecell_solver_append_string_sprintf(
+                fc_solve_append_string_sprintf(
                     app_str,
                     "%-16s        H-%1s C-%1s D-%1s S-%1s\n",
                     str2,
@@ -950,19 +982,19 @@ char * freecell_solver_state_as_string(
             }
             else
             {
-                freecell_solver_append_string_sprintf(
+                fc_solve_append_string_sprintf(
                     app_str,
                     "%s\n", str2
                     );
             }
-            freecell_solver_append_string_sprintf(
+            fc_solve_append_string_sprintf(
                 app_str,
                 "%s\n", str3
                 );
         }
         for(;a<decks_num;a++)
         {
-            freecell_solver_append_string_sprintf(
+            fc_solve_append_string_sprintf(
                 app_str,
                 "%-16s        H-%1s C-%1s D-%1s S-%1s\n",
                 "",
@@ -972,7 +1004,7 @@ char * freecell_solver_state_as_string(
                 decks[a*4+3]
                 );
         }
-        freecell_solver_append_string_sprintf(
+        fc_solve_append_string_sprintf(
             app_str,
             "%s",
             "\n\n"
@@ -980,9 +1012,9 @@ char * freecell_solver_state_as_string(
 
         for(s=0;s<stacks_num;s++)
         {
-            freecell_solver_append_string_sprintf(app_str, "%s", " -- ");
+            fc_solve_append_string_sprintf(app_str, "%s", " -- ");
         }
-        freecell_solver_append_string_sprintf(
+        fc_solve_append_string_sprintf(
             app_str,
             "%s",
             "\n"
@@ -1003,14 +1035,14 @@ char * freecell_solver_state_as_string(
             {
                 if (card_num >= fcs_stack_len(*state, stack_locs[s]))
                 {
-                    freecell_solver_append_string_sprintf(
+                    fc_solve_append_string_sprintf(
                         app_str,
                         "    "
                         );
                 }
                 else
                 {
-                    freecell_solver_append_string_sprintf(
+                    fc_solve_append_string_sprintf(
                         app_str,
                         "%3s ",
                         fcs_card_perl2user(
@@ -1024,15 +1056,15 @@ char * freecell_solver_state_as_string(
                         );
                 }
             }
-            freecell_solver_append_string_sprintf(app_str, "%s", "\n");
+            fc_solve_append_string_sprintf(app_str, "%s", "\n");
         }
     }
     else
     {
-        freecell_solver_append_string_sprintf(app_str, "%s", "Foundations: ");
+        fc_solve_append_string_sprintf(app_str, "%s", "Foundations: ");
         for(a=0;a<decks_num;a++)
         {
-            freecell_solver_append_string_sprintf(
+            fc_solve_append_string_sprintf(
                 app_str,
                 "H-%s C-%s D-%s S-%s ",
                 decks[a*4],
@@ -1042,11 +1074,11 @@ char * freecell_solver_state_as_string(
                 );
         }
 
-        freecell_solver_append_string_sprintf(app_str, "%s", "\nFreecells: ");
+        fc_solve_append_string_sprintf(app_str, "%s", "\nFreecells: ");
 
         for(a=0;a<freecells_num;a++)
         {
-            freecell_solver_append_string_sprintf(
+            fc_solve_append_string_sprintf(
                 app_str,
                 "%3s",
                 fcs_card_perl2user(
@@ -1060,14 +1092,14 @@ char * freecell_solver_state_as_string(
             );
             if (a < freecells_num-1)
             {
-                freecell_solver_append_string_sprintf(app_str, "%s", " ");
+                fc_solve_append_string_sprintf(app_str, "%s", " ");
             }
         }
-        freecell_solver_append_string_sprintf(app_str, "%s", "\n");
+        fc_solve_append_string_sprintf(app_str, "%s", "\n");
 
         for(s=0;s<stacks_num;s++)
         {
-            freecell_solver_append_string_sprintf(app_str, "%s", ": ");
+            fc_solve_append_string_sprintf(app_str, "%s", ": ");
 
             len = fcs_stack_len(*state, stack_locs[s]);
             for(card_num=0;card_num<len;card_num++)
@@ -1081,15 +1113,15 @@ char * freecell_solver_state_as_string(
                     stack_card_,
                     display_10_as_t
                 );
-                freecell_solver_append_string_sprintf(app_str, "%s", stack_card_);
+                fc_solve_append_string_sprintf(app_str, "%s", stack_card_);
                 if (card_num < len-1)
                 {
-                    freecell_solver_append_string_sprintf(app_str, "%s", " ");
+                    fc_solve_append_string_sprintf(app_str, "%s", " ");
                 }
             }
-            freecell_solver_append_string_sprintf(app_str, "%s", "\n");
+            fc_solve_append_string_sprintf(app_str, "%s", "\n");
         }
     }
 
-    return freecell_solver_append_string_finalize(app_str);
+    return fc_solve_append_string_finalize(app_str);
 }
