@@ -1,7 +1,7 @@
 /*
  * state.h - header file for state functions and macros for Freecell Solver
  *
- * Written by Shlomi Fish (shlomif@vipe.technion.ac.il), 2000
+ * Written by Shlomi Fish ( http://www.shlomifish.org/ ), 2000
  *
  * This file is in the public domain (it's uncopyrighted).
  */
@@ -9,8 +9,8 @@
 
 #include "fcs_move.h"
 
-#ifndef __STATE_H
-#define __STATE_H
+#ifndef FC_SOLVE__STATE_H
+#define FC_SOLVE__STATE_H
 
 #ifdef __cplusplus
 extern "C" {
@@ -30,8 +30,8 @@ extern "C" {
  *
  ************/
 
-#define is_scan_visited(ptr_state, scan_id) (ptr_state->scan_visited[(scan_id)>>5] & (1 << ((scan_id)&((1<<(5))-1))))
-#define set_scan_visited(ptr_state, scan_id) { ptr_state->scan_visited[(scan_id)>>5] |= (1 << ((scan_id)&((1<<(5))-1))); }
+#define is_scan_visited(ptr_state_val, scan_id) (ptr_state_val->scan_visited[(scan_id)>>5] & (1 << ((scan_id)&((1<<(5))-1))))
+#define set_scan_visited(ptr_state_val, scan_id) { ptr_state_val->scan_visited[(scan_id)>>5] |= (1 << ((scan_id)&((1<<(5))-1))); }
 
 
 #ifdef DEBUG_STATES
@@ -143,8 +143,12 @@ typedef int fcs_locs_t;
         (state).stacks[(ds)].num_cards++;  \
     }
 
-#define fcs_duplicate_state(dest, src) \
-    (dest) = (src)
+#define fcs_duplicate_state(dest_key, dest_val, src_key, src_val) \
+    { \
+    (dest_key) = (src_key); \
+    (dest_val) = (src_val); \
+    (dest_val).key = &(dest_key); \
+    }
 
 #define fcs_put_card_in_freecell(state, f, card) \
     (state).freecells[(f)] = (card)
@@ -178,7 +182,7 @@ typedef int fcs_locs_t;
     ((state).talon[pos] = (card))
 #endif
 
-#define fcs_copy_stack(state, idx, buffer) {} 
+#define fcs_copy_stack(state_key, state_val, idx, buffer) {}
 
 #elif defined(COMPACT_STATES)    /* #ifdef DEBUG_STATES */
 
@@ -301,8 +305,12 @@ typedef char fcs_locs_t;
 #define fcs_push_stack_card_into_stack(state, ds, ss, sc) \
     fcs_push_card_into_stack((state), (ds), fcs_stack_card((state), (ss), (sc)))
 
-#define fcs_duplicate_state(dest, src) \
-    (dest) = (src)
+#define fcs_duplicate_state(dest_key, dest_val, src_key, src_val) \
+    { \
+    (dest_key) = (src_key); \
+    (dest_val) = (src_val); \
+    (dest_val).key = &(dest_key); \
+    }
 
 #define fcs_put_card_in_freecell(state, f, card) \
     (state).data[FCS_FREECELLS_OFFSET+(f)] = (card);
@@ -340,8 +348,8 @@ typedef char fcs_locs_t;
 #define fcs_flip_stack_card(state, s, c) \
     (fcs_card_set_flipped(fcs_stack_card((state),(s),(c)), ((fcs_card_t)0) ))
 
-#define fcs_copy_stack(state, idx, buffer) {} 
-    
+#define fcs_copy_stack(state_key, state_val, idx, buffer) {}
+
 #elif defined(INDIRECT_STACK_STATES) /* #ifdef DEBUG_STATES
                                         #elif defined(COMPACT_STATES)
                                       */
@@ -453,25 +461,27 @@ typedef struct fcs_struct_state_t fcs_state_t;
     (fcs_card_set_flipped(fcs_stack_card(state,s,c), ((fcs_card_t)0) ))
 
 
-#define fcs_duplicate_state(dest,src)           \
-    {                                           \
-        (dest) = (src);                         \
-        (dest).stacks_copy_on_write_flags = 0;    \
+#define fcs_duplicate_state(dest_key, dest_val, src_key, src_val) \
+    { \
+    (dest_key) = (src_key); \
+    (dest_val) = (src_val); \
+    (dest_val).key = &(dest_key); \
+    (dest_val).stacks_copy_on_write_flags = 0; \
     }
 
-#define fcs_copy_stack(state, idx, buffer) \
+#define fcs_copy_stack(state_key, state_val, idx, buffer) \
     {     \
-        if (! ((state).stacks_copy_on_write_flags & (1 << idx)))        \
+        if (! ((state_val).stacks_copy_on_write_flags & (1 << idx)))        \
         {          \
             int stack_len;      \
-            (state).stacks_copy_on_write_flags |= (1 << idx);       \
-            stack_len = fcs_stack_len((state).s,idx);     \
-            memcpy(&buffer[idx << 7], (state).s.stacks[idx], stack_len+1); \
-            (state).s.stacks[idx] = &buffer[idx << 7];     \
+            (state_val).stacks_copy_on_write_flags |= (1 << idx);       \
+            stack_len = fcs_stack_len((state_key),idx);     \
+            memcpy(&buffer[idx << 7], (state_key).stacks[idx], stack_len+1); \
+            (state_key).stacks[idx] = &buffer[idx << 7];     \
         }     \
     }
-            
-            
+
+
 typedef char fcs_locs_t;
 
 #endif /* #ifdef DEBUG_STATES -
@@ -479,6 +489,10 @@ typedef char fcs_locs_t;
           #elif defined INDIRECT_STACK_STATES
         */
 
+/* Commenting out so the API will be broken - we're now using key/value
+ * pairs.
+ * */
+#if 0
 struct fcs_struct_state_with_locations_t
 {
     fcs_state_t s;
@@ -519,18 +533,78 @@ struct fcs_struct_state_with_locations_t
      * */
     int scan_visited[MAX_NUM_SCANS_BUCKETS];
 #ifdef INDIRECT_STACK_STATES
-    /* 
+    /*
      * A vector of flags that indicates which stacks were already copied.
      * */
-    int stacks_copy_on_write_flags; 
+    int stacks_copy_on_write_flags;
 #endif
 };
 
 typedef struct fcs_struct_state_with_locations_t fcs_state_with_locations_t;
+#endif
 
+struct fcs_state_extra_info_struct
+{
+    fcs_locs_t stack_locs[MAX_NUM_STACKS];
+    fcs_locs_t fc_locs[MAX_NUM_FREECELLS];
+    fcs_state_t * key;
+    struct fcs_state_extra_info_struct * parent_val;
+    fcs_move_stack_t * moves_to_parent;
+    int depth;
+    /*
+     * This field contains global, scan-independant flags, which are used
+     * from the FCS_VISITED_T enum below.
+     *
+     * FCS_VISITED_VISITED - deprecated
+     *
+     * FCS_VISITED_IN_SOLUTION_PATH - indicates that the state is in the
+     * solution path found by the scan. (used by the optimization scan)
+     *
+     * FCS_VISITED_IN_OPTIMIZED_PATH - indicates that the state is in the
+     * optimized solution path which is computed by the optimization scan.
+     *
+     * FCS_VISITED_DEAD_END - indicates that the state does not lead to
+     * anywhere useful, and scans should not examine it in the first place.
+     * */
+    int visited;
+    /*
+     * The iteration in which this state was marked as visited
+     * */
+    int visited_iter;
+    /*
+     * This is the number of direct children of this state which were not
+     * yet declared as dead ends. Once this counter reaches zero, this
+     * state too is declared as a dead end.
+     * */
+    int num_active_children;
+    /*
+     * This is a vector of flags - one for each scan. Each indicates whether
+     * its scan has already visited this state
+     * */
+    int scan_visited[MAX_NUM_SCANS_BUCKETS];
+#ifdef INDIRECT_STACK_STATES
+    /*
+     * A vector of flags that indicates which stacks were already copied.
+     * */
+    int stacks_copy_on_write_flags;
+#endif
+};
 
-extern fcs_card_t freecell_solver_empty_card;
-#define fcs_empty_card freecell_solver_empty_card
+typedef struct fcs_state_extra_info_struct fcs_state_extra_info_t;
+
+typedef struct
+{
+    fcs_state_t s;
+    fcs_state_extra_info_t info;
+} fcs_state_keyval_pair_t;
+
+typedef struct {
+    fcs_state_t * key;
+    fcs_state_extra_info_t * val;
+} fcs_standalone_state_ptrs_t;
+
+extern fcs_card_t fc_solve_empty_card;
+#define fcs_empty_card fc_solve_empty_card
 
 
 #ifdef FCS_WITH_TALONS
@@ -564,14 +638,12 @@ extern fcs_card_t freecell_solver_empty_card;
 #endif
 
 
-extern void freecell_solver_canonize_state(
-    fcs_state_with_locations_t * state,
+extern void fc_solve_canonize_state(
+    fcs_state_extra_info_t * state_val,
     int freecells_num,
     int stacks_num
     );
 
-#define fcs_canonize_state(state,freecells_num,stacks_num) freecell_solver_canonize_state((state),(freecells_num),(stacks_num))
-
 #if (FCS_STATE_STORAGE != FCS_STATE_STORAGE_INDIRECT)
 
 #if (FCS_STATE_STORAGE != FCS_STATE_STORAGE_LIBREDBLACK_TREE)
@@ -580,20 +652,30 @@ typedef void * fcs_compare_context_t;
 typedef const void * fcs_compare_context_t;
 #endif
 
-extern int freecell_solver_state_compare(const void * s1, const void * s2);
-extern int freecell_solver_state_compare_equal(const void * s1, const void * s2);
-extern int freecell_solver_state_compare_with_context(const void * s1, const void * s2, fcs_compare_context_t context);
+extern int fc_solve_state_compare(const void * s1, const void * s2);
+extern int fc_solve_state_compare_equal(const void * s1, const void * s2);
+extern int fc_solve_state_compare_with_context(const void * s1, const void * s2, fcs_compare_context_t context);
+extern int fc_solve_state_extra_info_compare_with_context(const void * s1, const void * s2, fcs_compare_context_t context);
+
 #else
-extern int freecell_solver_state_compare_indirect(const void * s1, const void * s2);
-extern int freecell_solver_state_compare_indirect_with_context(const void * s1, const void * s2, void * context);
+extern int fc_solve_state_compare_indirect(const void * s1, const void * s2);
+extern int fc_solve_state_compare_indirect_with_context(const void * s1, const void * s2, void * context);
 #endif
 
 #ifdef FCS_WITH_TALONS
 extern int fcs_talon_compare_with_context(const void * s1, const void * s2, fcs_compare_context_t context);
 #endif
 
-extern fcs_state_with_locations_t freecell_solver_initial_user_state_to_c(
+enum FCS_USER_STATE_TO_C_RETURN_CODES
+{
+    FCS_USER_STATE_TO_C__SUCCESS = 0,
+    FCS_USER_STATE_TO_C__PREMATURE_END_OF_INPUT
+};
+
+int fc_solve_initial_user_state_to_c(
     const char * string,
+    fcs_state_t * key,
+    fcs_state_extra_info_t * out_state_val,
     int freecells_num,
     int stacks_num,
     int decks_num
@@ -606,8 +688,8 @@ extern fcs_state_with_locations_t freecell_solver_initial_user_state_to_c(
     );
 
 
-extern char * freecell_solver_state_as_string(
-    fcs_state_with_locations_t * state,
+extern char * fc_solve_state_as_string(
+    fcs_state_extra_info_t * state_val,
     int freecells_num,
     int stacks_num,
     int decks_num,
@@ -615,8 +697,18 @@ extern char * freecell_solver_state_as_string(
     int canonized_order_output,
     int display_10_as_t
     );
-extern int freecell_solver_check_state_validity(
-    fcs_state_with_locations_t * state,
+
+enum FCS_STATE_VALIDITY_CODES
+{
+    FCS_STATE_VALIDITY__OK = 0,
+    FCS_STATE_VALIDITY__EMPTY_SLOT = 3,
+    FCS_STATE_VALIDITY__EXTRA_CARD = 2,
+    FCS_STATE_VALIDITY__MISSING_CARD = 1,
+    FCS_STATE_VALIDITY__PREMATURE_END_OF_INPUT = 4,
+};
+
+extern int fc_solve_check_state_validity(
+    fcs_state_extra_info_t * state_val,
     int freecells_num,
     int stacks_num,
     int decks_num,
@@ -640,4 +732,4 @@ enum FCS_VISITED_T
 };
 
 
-#endif /* __STATE_H */
+#endif /* FC_SOLVE__STATE_H */
